//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                               SonicR Solid Dragon-Trend.mq4                               |
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2012 traderathome"
#property link      "email: traderathome@msn.com"

/*---------------------------------------------------------------------------------------------
Overview, _v2:

This indicator displays both the SonicR Dragon and the SonicR Trend.  The SonicR Dragon is 
color filled and is based upon 34 EMA averaging of close prices with high/low prices defining 
outer edges.  The SonicR Trend is based upon 89 EMA averaging of close prices.  

About color filling the Dragon-
   Histogram bars and EMA lines are used in the construction of the color fill.  Zooming in 
   on a chart requires these components be made wider to prevent spaces appearing between 
   them.  Zooming out on a chart requires these components be made narrower to prevent the 
   Dragon from being too wide.  The Dragon can be zoom adjusted automatically or manually.
   
Automatic_Zoom-
   This feature will have widths automatically adjust, with the next arriving tick, to what 
   is appropriate for the chart zoom setting.  When MT4 is first started the bar widths are 
   incorrect until the first tick arrives.  If ticks are slow coming, or if there is no live
   feed (weekend/holiday), you can force the width adjustment by switching the chart TF.  
      
   The widths are set by code based on encountered "WindowBarsPerChart( )".  As you zoom
   out, there are more.  As you zoom in there are less.  This might be affected by the 
   width capacity of the monitor.  Extra wide monitors, if showing more bars per chart
   on all zoom settings than what the code expects, can result in narrower bars displayed 
   than intended.  This is because the additional bars encountered are interpreted by the 
   code as meaning the chart has been zoomed out an extra step or more, and the code displays
   the narrower bar widths that would otherwise be appropriate.  Downsizing charts presents a 
   problem also, causing bar widths that are too wide because the fewer bars encountered are 
   interpreted by the code as meaning the chart has been zoomed in an extra step or more.  It
   therefore displays the wider bars that would otherwise be appropriate.  So, in cases of an 
   extra wide monitor or downsizing of charts, undesirable results can occur and you should 
   use the manual width adjustment.  
            
Manual_Zoom_01234-
   For reasons stated in #2 above, or if you tend to use only the standard zoom setting, or 
   some other single zoom setting, manually selecting the zoom setting is the best choice.  
   Selecting "0" sets widths appropriate for the two chart zoom out/"-" settings that are
   below the standard chart zoom setting.  Selecting "1" sets width appropriate for the Mt4
   standard/default chart zoom setting.  Apply "2,3, or 4" for zooming in/"+" on a chart from
   the standard zoom setting to get the wider bars that are appropriate. 
   
Optional Displays of the Dragon and Trend-
   The Dragon and Trend from one TF chart can be displayed on a different TF chart. 
   1. The M5 display of the Dragon and Trend to be applied to the M15 TF chart.
   2. The M15 display of the Dragon and Trend be applied to the M30 and H1 TF charts.
   3. The H1 display of the Dragon and Trend be applied to the H4 TF chart.
   By placing the cursor on either the Dragon or Trend and a window appears identifying  
   this indicator.  Right click the cursor and another window appears identifying the
   configurations of the Dragon and Trend that have been applied to the chart.  Brokers 
   that have short Sunday sessions lack data to have such "transferred" configurations 
   drawn seamlessly.  This indicator identifies when that is the case and compensates by 
   filling with data that is the average of that which is before and after an interruption 
   of the draw.  If this not take place automatically when a chart is loaded, or selected, 
   simply changing the chart TF once should refresh the draw correctly.
           
You can select a range of TFs for the display of this indicator, so it automatically will not 
display on a chart TF outside that range.  The indicator can be turned on/off without having 
to remove it from the chart, thereby preserving your chart settings.  

Finally, be sure that chart Properties/Common tab/"Chart on foreground" is checked, so that 
the price display is on top of the Solid Dragon.

Changes from previous release 04-04-2012 to current release 05-01-2012:
01 - Compensate for data dropouts interrupting the draw of transferred configurations.

                             
                                                                   - Traderathome, 05-01-2012
----------------------------------------------------------------------------------------------
Suggested Colors             White Chart      Black Chart       
 
indicator_color1-4           C'221,238,255'   C'40,45,100'   
indicator_color5             SlateBlue        DarkSlateBlue 
indicator_color6             Magenta          C'215,0,215'                                                                                                                                                           
---------------------------------------------------------------------------------------------*/


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+ 
#property indicator_chart_window
#property indicator_buffers  6

//Dragon-
#property indicator_color1   C'40,45,100'     //high area
#property indicator_color2   C'40,45,100'     //low area 
#property indicator_color3   C'40,45,100'     //top line     
#property indicator_color4   C'40,45,100'     //bot line  
#property indicator_color5   C'100,100,100'   //cntr line 
#property indicator_width1   4
#property indicator_width2   4
#property indicator_width3   5
#property indicator_width4   5
#property indicator_width5   1
#property indicator_style1   STYLE_SOLID
#property indicator_style2   STYLE_SOLID
#property indicator_style3   STYLE_SOLID
#property indicator_style4   STYLE_SOLID 
#property indicator_style5   STYLE_SOLID 

//Trend-
#property indicator_color6   C'215,0,215'    
#property indicator_width6   1
#property indicator_style6   STYLE_SOLID

//global external inputs
extern bool   Indicator_On                    = true;
extern bool   Automatic_Zoom                  = false;
extern int    Manual_Zoom_01234               = 1;
extern bool   Dragon_On                       = true;
extern bool   Trend_On                        = true; 
extern int    Display_Min_TF                  = 1;
extern int    Display_Max_TF                  = 43200;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";

extern string __                              = "";
extern string Part_2                          = "Dragon-Trend Display Options:";
extern bool   M5_On_M15                       = false;
extern bool   M15_On_M30                      = false;
extern bool   M15_On_H1                       = false;
extern bool   H1_On_H4                        = false; 

//Global Buffers and Variables
//General-
bool    FLAG_deinit;

//Dragon-
int     Wide_Bar_Width,TopEdge,BotEdge,i,counted_bars,limit,BarShift;
double  DragonHigh[], DragonLow[], DragonTop[], DragonBot[], DragonCntr[];
string  Dragon_MA_String              = "EMA";
int     Dragon_MA_Type                = 1;
int     Centerline                    = 0;
int     Dragon_MA_Period              = 34;
bool    Dragon_Exp_vs_Simple          = true;

//Trend-
double  Trend[];
string  Trend_MA_String               = "EMA";
int     Trend_MA_Type                 = 1;
int     Trend_MA_Price                = 0;
int     Trend_MA_Period               = 89;
bool    Trend_Exp_vs_Simple           = true;  

/*General notes:
MAType   = 0=SMA,1=EMA,2=SMMA,3=LWMA
MAPrice  = 0=CLOSE,1=OPEN,2=HIGH,3=LOW,4=MEDIAN,5=PP,6=WEIGHT
*/

//Automatic Zoom Parameters
string note1                           = "Auto-Width_Adjustments:";
string note2                           = "Wide monitors may require";
string note3                           = "WindowsBarsPerChart be increased";
string note4                           = "for these chart zoom settings:";
int    Zoom_Out_2_Bars                 = 1200;
int    Zoom_Out_1_Bars                 = 500;
int    Zoom_Std_Bars                   = 200;
int    Zoom_In_1_Bars                  = 100;
int    Zoom_In_2_Bars                  = 60;

//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+      
int init()
  {
  FLAG_deinit = false;

  //Automatically Adjust Width      
  if(Automatic_Zoom)   
    {   
          if (WindowBarsPerChart( ) >= Zoom_Out_2_Bars) {Wide_Bar_Width=1; TopEdge=1; BotEdge=1;} 
    else {if (WindowBarsPerChart( ) >= Zoom_Out_1_Bars) {Wide_Bar_Width=2; TopEdge=1; BotEdge=1;} 
    else {if (WindowBarsPerChart( ) >= Zoom_Std_Bars)   {Wide_Bar_Width=3; TopEdge=3; BotEdge=3;} 
    else {if (WindowBarsPerChart( ) >= Zoom_In_1_Bars)  {Wide_Bar_Width=5; TopEdge=7; BotEdge=7;} 
    else {if (WindowBarsPerChart( ) >= Zoom_In_2_Bars)  {Wide_Bar_Width=9; TopEdge=14; BotEdge=14;} 
    else {Wide_Bar_Width=17; TopEdge=26; BotEdge=26;} }}}}
    }
  else //Manually Adjust Width
    {
          if (Manual_Zoom_01234 <= 0) {Wide_Bar_Width = 2; TopEdge = 1;BotEdge = 1;}
    else {if (Manual_Zoom_01234 == 1) {Wide_Bar_Width = 3; TopEdge = 3;BotEdge = 3;}
    else {if (Manual_Zoom_01234 == 2) {Wide_Bar_Width = 5; TopEdge = 7;BotEdge = 7;}
    else {if (Manual_Zoom_01234 == 3) {Wide_Bar_Width = 9; TopEdge = 14;BotEdge = 14;}
    else {Wide_Bar_Width = 17; TopEdge = 26;BotEdge = 26;} }}}
    }        
  
  //Indicators    
  if(Dragon_On)
    {              
    SetIndexBuffer(0, DragonHigh);
    SetIndexBuffer(1, DragonLow);
    SetIndexBuffer(2, DragonTop);
    SetIndexBuffer(3, DragonBot);
    SetIndexBuffer(4, DragonCntr);                   
    SetIndexStyle(0, DRAW_HISTOGRAM, 0, Wide_Bar_Width);    
    SetIndexStyle(1, DRAW_HISTOGRAM, 0, Wide_Bar_Width);    
    SetIndexStyle(2, DRAW_LINE, 0, TopEdge); 
    SetIndexStyle(3, DRAW_LINE, 0, BotEdge);
    SetIndexStyle(4, DRAW_LINE); 
    }                                   
  if(Trend_On) 
    {    
    SetIndexBuffer(5, Trend);                     
    SetIndexStyle(5, DRAW_LINE);          
    }
      
  //Indicator ShortName    
  if(Dragon_On && Trend_On)
    {
    if(M5_On_M15 && Period() == 15) 
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( M5  "+Dragon_MA_Period+" "+Dragon_MA_String
      +",  M5  "+Trend_MA_Period+" "+ Trend_MA_String+" ) ");
      }
    else if((M15_On_M30 && Period() == 30) ||
      (M15_On_H1 && Period() == 60))
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( M15  "+Dragon_MA_Period+" "+Dragon_MA_String
      +",  M15  "+Trend_MA_Period+" "+ Trend_MA_String+" ) ");
      } 
    else if(H1_On_H4 && Period() == 240)
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( H1  "+Dragon_MA_Period+" "+Dragon_MA_String
      +",  H1  "+Trend_MA_Period+" "+ Trend_MA_String+" ) ");
      }     
    else
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( "+Dragon_MA_Period+" "+Dragon_MA_String
      +",  "+Trend_MA_Period+" "+ Trend_MA_String+" ) ");
      }         
    }
  else if(Dragon_On && !Trend_On)
    {
    if(M5_On_M15 && Period() == 15)
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( M5  "+Dragon_MA_Period+" "+Dragon_MA_String+" ) ");
      }
    else if((M15_On_M30 && Period() == 30) ||
      (M15_On_H1 && Period() == 60))
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( M15  "+Dragon_MA_Period+" "+Dragon_MA_String+" ) ");
      } 
    else if(H1_On_H4 && Period() == 240)
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( H1  "+Dragon_MA_Period+" "+Dragon_MA_String+" ) ");
      }
    else
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( "+Dragon_MA_Period+" "+Dragon_MA_String+" ) ");
      }           
    }    
  else if(Trend_On && !Dragon_On)
    {
    if(M5_On_M15 && Period() == 15)
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( M5  "+Trend_MA_Period+" "+Trend_MA_String+" ) ");
      }
    else if((M15_On_M30 && Period() == 30) ||
      (M15_On_H1 && Period() == 60))
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( M15  "+Trend_MA_Period+" "+Trend_MA_String+" ) ");
      }
    else if(H1_On_H4 && Period() == 240)
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( H1  "+Trend_MA_Period+" "+Trend_MA_String+" ) ");
      }
    else
      {
      IndicatorShortName("SonicR Solid Dragon-Trend ( "+Trend_MA_Period+" "+ Trend_MA_String+" ) ");
      }           
    }  
                                         
                                         
  return(0);
  }

//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+      
int deinit()
  {      
  return(0);
  }

//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+     
int start()
  {
  //If indicator is "Off" or chart TF is out of range deinitialize only once, not every tick.  
  if((!Indicator_On) || ((Period() < Display_Min_TF) || (Period() > Display_Max_TF)))    
    {
    if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
    return(0);
    }

  //Otherwise indicator is "On" & chart TF is in display range, so proceed.   
  if(Automatic_Zoom) {init();}
  else {deinit(); FLAG_deinit = false;}
                       
  //Calculate MAs   
  //check for possible errors
  counted_bars = IndicatorCounted();
  if(counted_bars < 0)  return(-1); 
     
  //last counted bar will be recounted
  if(counted_bars > 0) counted_bars--;    
  limit = Bars - counted_bars;

  for(i = limit - 1; i >= 0; i--)
    {
    if(Dragon_On)
      {
      if(M5_On_M15 && Period() == 15)  
        { 
        BarShift = iBarShift(NULL,5,Time[i],true); 
        DragonHigh[i]   = iMA(NULL,5,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_HIGH,BarShift);
        DragonLow[i]    = iMA(NULL,5,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_LOW,BarShift);       
        DragonTop[i]    = iMA(NULL,5,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_HIGH,BarShift); 
        DragonBot[i]    = iMA(NULL,5,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_LOW,BarShift);
        DragonCntr[i] = iMA(NULL,5,Dragon_MA_Period,0,Dragon_MA_Type,Centerline,BarShift);   
        }       
      else if((M15_On_M30 && Period() == 30)  ||  (M15_On_H1 && Period() == 60))
        { 
        BarShift = iBarShift(NULL,15,Time[i],true); 
        DragonHigh[i]   = iMA(NULL,15,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_HIGH,BarShift);
        DragonLow[i]    = iMA(NULL,15,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_LOW,BarShift);       
        DragonTop[i]    = iMA(NULL,15,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_HIGH,BarShift); 
        DragonBot[i]    = iMA(NULL,15,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_LOW,BarShift); 
        DragonCntr[i] = iMA(NULL,15,Dragon_MA_Period,0,Dragon_MA_Type,Centerline,BarShift); 
        }
      else if (H1_On_H4 && Period() == 240) 
        { 
        BarShift = iBarShift(NULL,60,Time[i],true); 
        DragonHigh[i]   = iMA(NULL,60,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_HIGH,BarShift);
        DragonLow[i]    = iMA(NULL,60,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_LOW,BarShift);       
        DragonTop[i]    = iMA(NULL,60,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_HIGH,BarShift); 
        DragonBot[i]    = iMA(NULL,60,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_LOW,BarShift); 
        DragonCntr[i] = iMA(NULL,60,Dragon_MA_Period,0,Dragon_MA_Type,Centerline,BarShift); 
        }                 
      else    
        {              
        BarShift = iBarShift(NULL,NULL,Time[i],true); 
        DragonHigh[i]   = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_HIGH,BarShift);   
        DragonLow[i]    = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_LOW,BarShift);                    
        DragonTop[i]    = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_HIGH,BarShift);                           
        DragonBot[i]    = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_MA_Type,PRICE_LOW,BarShift);
        DragonCntr[i] = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_MA_Type,Centerline,BarShift);
        }
      //Brokers with short Sunday sessions lack necessary data to make Dragon continuous, so when
      //there is a data "dropout" (current value<half previous value), fill with average data: 
      if(DragonHigh[i+1]< DragonHigh[i+2]/2){DragonHigh[i+1]= (DragonHigh[i+2]+ DragonHigh[i])/2;}
      if(DragonLow[i+1]< DragonLow[i+2]/2){DragonLow[i+1]= (DragonLow[i+2]+ DragonLow[i])/2;}
      if(DragonTop[i+1]< DragonTop[i+2]/2){DragonTop[i+1]= (DragonTop[i+2]+ DragonTop[i])/2;}
      if(DragonBot[i+1]< DragonBot[i+2]/2){DragonBot[i+1]= (DragonBot[i+2]+ DragonBot[i])/2;}      
      if(DragonCntr[i+1]< DragonCntr[i+2]/2){DragonCntr[i+1]= (DragonCntr[i+2]+ DragonCntr[i])/2;}                 
      }
         
    if(Trend_On)   
      {
      if(M5_On_M15 && Period() == 15)   
        {
        BarShift = iBarShift(NULL,5,Time[i],true);                 
        Trend[i]   = iMA(NULL,5,Trend_MA_Period,0,Trend_MA_Type,Trend_MA_Price,BarShift); 
        } 
      else if((M15_On_M30 && Period() == 30)  ||  (M15_On_H1 && Period() == 60))
        {
        BarShift = iBarShift(NULL,15,Time[i],true);                 
        Trend[i]   = iMA(NULL,15,Trend_MA_Period,0,Trend_MA_Type,Trend_MA_Price,BarShift);   
        }
      else if (H1_On_H4 && Period() == 240) 
        {
        BarShift = iBarShift(NULL,60,Time[i],true);                 
        Trend[i]   = iMA(NULL,60,Trend_MA_Period,0,Trend_MA_Type,Trend_MA_Price,BarShift);
        }     
      else
        {   
        BarShift = iBarShift(NULL,NULL,Time[i],true);                 
        Trend[i]   = iMA(NULL,NULL,Trend_MA_Period,0,Trend_MA_Type,Trend_MA_Price,BarShift);             
        }
      //Brokers with short Sunday sessions lack necessary data to make Trend continuous, so when
      //there is a data "dropout" (current value<half previous value), fill with average data:
      if(Trend[i+1]< Trend[i+2]/2){Trend[i+1]= (Trend[i+2]+ Trend[i])/2;}         
      }   
    }                
                                                                                                                  
  return(0);
  }

//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+       

