//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                SonicR Control Panel.mq4                                   | 
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2012 traderathome"
#property link      "email: traderathome@msn.com" 

/*---------------------------------------------------------------------------------------------
Overview, _v2:

This SonicR Control Panel indicator has important changes from the last release 07/16/2011.  
A list of the changes can be found at the bottom of this overview.  Most notably, the SonicR
Clock Panel has been integrated into this indicator. And also worth mentioning up front is the
TzPivots feature and the market open vLines feature have been added back with new coding that
eliminates the need for the inputs "DST_Server_GMT_Offset" and "DST_Zone".  The sophisticated 
features of this indicator are more fully automated than ever before and more simple to use.

Remarks On the "Preface" Inputs-
You only need to be concerned with this section if you have selected to display TzPivots or 
vLines at a time when there is no live data feed from your broker server (weekends/holidays). 
Broker time is required by the code that features TzPivots and vLines.  When data feed is live
the broker time is available.  When data feed is not live it is not available and manually
input correct information must be used to compensate, hence, the two inputs in this section.
Do the following to get the correct "Broker_GMT_Offset" value to input.  While data feed is
live, temporarily add "GMT" and "Broker" to the Clock if the display does not already show
"G" and "B" in the labels.  Compare the +/- hours time difference between GMT and Broker time.
For example, if GMT is 11pm and broker time is 2am, then the difference is "3" hours.  if GMT 
is 1am and broker time is 9pm, then the difference is "-4" hours.  Enter the time difference as 
the "Broker_GMT_Offset".  TzPivots and vLines will now function properly when there is no live 
feed if you manually apply the "Broker_GMT_Offset" by making "Apply_it_manually" equal "true".  
When the data feed is live be sure "Apply_it_manually" equals "false", because in this mode the 
indicator adjusts for broker server DST changes automatically when they actually occur twice 
yearly.  Obviously, you must check your "Broker_GMT_Offset" input twice yearly when there is a 
DST changeover on the broker server, and correct the value as necessary.  The default is set
to "0" for a broker server (broker time) at GMT.  If your broker is at GMT there are no DST
changeovers, and this setting will not require changes twice yearly.

This indicator places a colored panel in the upper left corner of the chart.  It consists of
three sections-
   1. A market section containing: (1) the pair symbol and time frame, (2) the spread, 
      (3) the daily average range, and the current range for the session), (4) the time to the 
      next candle, and (5) the market bid price.  
   2. The "SonicR" logo.
   3. The Time Zone Clock.

This indicator can be used for both forex and non-forex items. In the Forex mode, you can show 
either 4 digit or 5 digits.  If your platform is 4 digit and you select the 5 digit display, 
a "0" is added to displayed numbers. If it is a 5 digit platform and you select to display 4 
digits, numbers are rounded off to 4 digits, except for the Bid display, which just drops the
fifth digit and will not increment/decrement until the next whole 4 digit number is reached. 

This indicator is a multi-function indicator, providing: 
   1. A special bid line.
   2. A background of level lines dividing the distance between whole numbers into quarters. 
   3. Daily or Fibonacci pivots, including mid-pivot lines.
   4. Average Range H/L lines for both the day and week.
   5. Day separators. 
   6. Vertical lines marking the London Open, the New York Open, and the London Close. 
   7. A time zone clock that displays seven time zones of your selection.

1. Optional Bid Line-
You can select to place your own Bid Ray Line on the chart, which you can vary in thickness 
and color.  The Bid Ray Line ends at the current candle, or at your specified number of TFs 
beyond.  It terminates with a Bid Dot that can be sized and that changes color(grn/red) as 
the bid price changes(up/dn).   To display only your own Bid Ray Line, right click on the 
chart, select Chart Properties/Colors and make the Grid color = "None".  Otherwise, the MT4 
bid line is beneath your own, and visible from the end of your line to the chart edge.

2. Optional Level Lines-
This indicator places level lines on your chart at key levels between whole numbers (00, 25,
50, 75 and 00 again).  The number of "sets" of these lines above and below the current 
price range "set" is determined by hard code.  But "Override_Line_Sets_Displayed" allows for 
change.  For example, if "1" is entered, then one set of lines is displayed for the current 
range.  If "2" is entered, then one set of lines is displayed for the current range and a 
second set is displayed both above and below the current range.  An input of "0" or less will 
default to the hard code.  The lines can individually be of any available color and style and 
thickness.  The half and quarter number levels can be turned On/Off individually.   

3. Optional Pivots -
   A. Main Pivot Line
      You can select to display the main pivot line for the day.  It can be constructed using 
      two lines.  One can be a wide background line of a highlighting color (Example: Yellow) 
      and the other can be a narrow top line of a darker color (Example: Gray).
   B. Support & Resistance Lines-
      You can select to display the SR lines using either the Daily or Fibonacci formulas.  
      The Fibonacci displays five levels.  Normally, the Daily displays only three levels, but 
      is extended to include five levels.  
   C. Broker or Time Zone Shifted Pivots
      You can display your broker server pivots, or you can select to display pivots based on  
      a different time zone.  New coding fully automates the complexities of TzPivots, such 
      as timing the hour shift for DST twice yearly, depending on when your broker does that.
      Your choice to use TzPivots requires only two simple inputs-
      I.  "TzPivots_On" 
          Set this equal to "true".
      II. "TzPivots_GMT_Offset" 
          This is your choice of time zone location relative to GMT in +/- hours (examples: 
          for GMT use "0", for one time zone east of GMT use "1").  This is a geographical 
          zone described by an offset from GMT remaining fixed regardless of DST consideratons.
          TzPivots are shown with an (*) in the label as a visual reminder.  Understand that 
          by choosing TzPivots when the next TzPivots day occurs during your broker server day, 
          the TzPivots lines will suddenly change.       
      
4. Optional Range High/Low Lines for the Day and Week-
You can select to display two horizontal lines, one for the computed range high target, and 
the other for the computed range low target, for both the day and the week.  The range is 
based on the averaging period you select.  The special range averaging process for the Day
range skips any brief Sunday sessions of some brokers, which would drag the average down.  
   A. Two conditions determine where these lines appear....
      1. Condition #1 - the Day/Week range has not exceeded the computed average range. 
         * The RDH/RWH line is the computed average range distance above the session Low.
         * The RDL/RWL line is the computed average range distance below the session high.  
         * The lines will move as new highs/lows are achieved during the session.  
         * This display shows how far price can move in either direction before exceeding
           the computed average range.  
      2. Condition #2 - the session range has exceeded the computed average range. 
         * RDH/RWH is the day/week high, if it came before the day/week low, and RDL/RWL is 
           the computed average range below RDH/RWH.
         * RDL/RWL is the day/week low, if it came before the day/week high, and RDH/RWH is 
           the computed average range above RDL/RWL.
         * The lines lock in these postions for the balance of the session (day/week).
         * This display shows the first breakout of the range, and subsequent activity. 
   B. The purpose of the range lines is to provide a perspective on how far PA might move 
      during the trading day or week.  This can be of some guidance in selecting where to 
      exit a trade.  A swing trader might pay more attention to the RWH/RWL lines, and hold
      a trade into oncoming days.  However, a day trader looking for quick profits and 
      limited exposure to market price swings might pay more attention to the RDH/RDL lines.
 
5. Optional Separators-
You can apply day separators to your chart with this option.  It lets you select color 
and line styles.  You can elect to show separators just for today and/or for any number 
of days previously.

6. Optional Vertical Time Lines-
You can select to display vertical time lines for the London Open, New York Open, and 
the the London Close.  New coding in this release properly places these vLines without 
any special inputs required from the user because it determines all year around the GMT 
offsets of the broker server, London and NYC (EST).  And the coding takes into account 
any changes at these locations due to DST/nonDST changeovers when they are implemented, 
even if at different times at each location.  This is the information required to always 
have the vLines displayed on your broker server chart at the places representing the 
proper local times in both London and NYC.

7. Time Zone Clock-
You can select to display seven time zones on the Clock Panel from a complete list of 
time zones provided.  If any of the time zones being displayed happens to coincide with
the broker/server time, your local time, or GMT, then extra labels will automatically
appear designating:  (B), (L), (G), (B,L), (B,G), (L,G) and (B,L,G).  With these labels
you do not have to waste lines to also display the Broker, Local or GMT time zones.
   
8. Addtitional Remarks On Displaying Horizontal Lines-
You can select to subordinate horizontal lines and/or their labels.  The horizontal line
groups can be displayed differently, one from another, using display numbers 0-4 in the 
section "Horizontal Lines Position Settings:".  The horizontal line groups are: whole
number Levels, half number Levels, quarter number Levels, central pivot, S&R pivots, and 
Range H/L lines (day/week).
  
   "0" draws the lines across the chart.  
   "1" draws the lines from chart left thru the current session.  
   "2" draws the lines only thru the current session.  
   "3" draws the lines from the start of the current session to chart right.  
   "4" draws the lines from the current candle to chart right. 
  
This last option ("4") allows you to shift the start of the lines (and labels) further away 
from the current candle using a numerical input in the section "Display_4_Shift_Line_Right".
  
   "0" no shift right hard coded position is selected.  
   "1" min. shift right (+) hard coded position is selected.  
   "2" next shift right (++) hard coded position is selected.   
   "3" next shift right (+++) hard coded position is selected.  
   "4" next shift right (++++) hard coded position is selected.   
   "5" most shift right (+++++) hard coded position is selected. 
  
Here is what happens when you choose to start lines further right from the last candle.
The feature to start lines further right from the current candle uses a specific time 
(date) for the lines to begin.  Let us say you want the horizontal lines to start one inch 
in from the right chart border.  What date is that?   It will be a near time date for a 
chart that is zoomed all the way in (widest candles).  It will be a date further out in 
time for a chart that is zoomed all the way out (narrowest candles). The specific date for 
each chart zoom setting is determined by the number of chart TFs between the last candle to 
the start of the line. The widest candle chart would require the fewest TFs to reach that 
date, and the narrowest candle chart would require many more TFs to reach that date.  This 
all assumes you want the lines to start the same distance in from the right chart border, 
no matter what zoom setting you have the chart set at.

Things have been made simple so you do not have to figure out what are the different TFs 
to use on different zoomed charts to achieve a specific shifted line position that will be
the same indented space from the right chart border on all chart zoom settings.  Hard code
gets the WindowBarsPerChart() to determine what zoom setting the chart is on.  The numbers 
used to determine each zoom setting are part of the external inputs section, because you
might have to increase them if you are using a monitor that is so wide it might be causing 
problems.  Once the zoom setting is found, hard code uses your selection out of the choices 
provided in "Display_4_Shift_Line_Right" to set the starting point of the lines.  However, 
keep in mind that if an extra wide monitor is used, more WindowBarsPerChart( ) might be 
encountered on all zoom settings than has been anticipated in the code, possibly causing 
this feataure to work incorrectly.  This will require you to increase the "Zoom....Bars" 
inputs by "trial and error" until you achieve proper display results on all zoom settings.


List of Changes From Last Release (07/16/2011) of the SonicR Chart Panel:
01 - Added back the time zone shifted pivots, this time including mid-pivots and fibonacci
     pivots.  New coding automatically determines broker server time shifts, replaces 
     the previous DST coding, and eliminates the need to input a "DST_Zone".
02 - Added back the vlines for market open times (Lc, Ny, Lc).  New coding automatically 
     determines their proper placement all year around regardless of DST considerations.
     Included is the ability to choose whether or not to display the vLine labels.
03 - Fixed #3 line display choice "Current session to right" so that it now works on all 
     TFs, not just those below H4.
04 - Added Week range lines with Max/Min TF to show, minor label placement adjustment for 
     slightly wide week range labels, and reordered codes for Week range, Day range, and
     Pivots to construct in that order.
05 - Fixed code for the display of ranges lines so now the Day range lines are correct on 
     Weekly and Monthly charts, and the Week range lines are correct on Monthly charts.
06 - Added subordination control of horizontal lines/labels (Levels always subordinated).     
07 - Fixed levels to properly display for additional symbols.      
08 - Automated the adjustment of "Display_4_Shift_Line_Right" when changing chart zoom, 
     provided choice of five shift right positions that can be used for indenting the 
     different catagories of horizontal lines/labels when using the "4" display choice, and
     included automatic indent shifting left of range lines for when pivot S&R lines are 
     turned on while range lines are set to same shift right position.          
09 - Housekeeping changes to levels and pivots labels, including making central pivot label 
     the same for Daily & Fib formula, removal of MaxRight override label positioning, and
     faster execution of labeling code in general.
10 - Recoded display of symbol in panel to strip broker added characters.
11 - Incorporated the separate SonicR Clock Panel indicator into this indicator with new
     coding automatically highlighting time zones matching your Broker and Local time.
12 - Revised the External Inputs accordingly.     

Changes from previous release 04-04-2012 to current release 05-01-2012:
01 - Revised the code using imported time zone information to get broker, London, and NYC
     offsets from GMT (used for TzPivots and vLines placement) to fix errant vLines at
     transitions between AM and PM hours. 
02 - Revised code for current session vLines to preclude stacking of these lines at the
     beginning of "tail end" Sunday sessions that some brokers have.
03 - Improved coding of daily range value.        
            
                                                                   - Traderathome, 05-01-2012                                                                     
----------------------------------------------------------------------------------------------
Acknowlegements:
CaveMan - who generously devoted his time and programming skills to produce the TzPivots time 
          zone shift coding, designed to handle any situation involving gaps between trading 
          days and/or partial trading days.
Domas4  - "THANKS!" are owed to Domas4 for providing price labeling code compatible with non-
          forex items.           
Kent    - for his P4L clock code to import and use time zone information. 

----------------------------------------------------------------------------------------------
Suggested Settings:          White Chart              Black Chart 

Panel Section-        
Panel_Background_Color       C'238,238,255'           C'25,25,25'
Symbol_And_TF_Color          Black                    C'208,208,208'
Spread_Color                 Black                    C'150,150,150'
Range_Color                  Black                    C'150,150,150'
Candle_Time_Color            Black                    C'150,150,150'
PriceLabel_UP_Color          ForestGreen              C'46,188,46'
PriceLabel_DOWN_Color        Crimson                  C'234,0,0'
sonicS                       C'44,44,255'             C'4,162,255'
sonicO                       C'111,111,255'           C'0,115,183'
SonicN                       C'170,170,255'           C'0,75,121' 
SonicI                       C'252,154,156'           C'138,0,0'
sonicC                       C'239,61,105'            C'176,0,0'
sonicR                       C'240,21,77'             C'221,0,0' 

Bid Line Section-
Bid_Ray_Color                Black                    C'100,100,100' 
Bid_Dot_Up_Color             LimeGreen                Lime
Bid_Dot_Dn_Color             OrangeRed                Red 

Levels Section-
Line_Labels_Color            MidnightBlue             DarkGray    
Whole_Number_Color           C'235,235,252'           C'41,33,73'         
Half_Number_Color            C'210,251,210'           C'0,53,0'              
Quarter_Number_Color         C'242,238,227'           C'43,33,19  

Pivots Section-                              
DPV1_Background_Color        Yellow                   CLR_NONE   
DPV2_Top_Color               Black                    C'230,216,0' 
Pivot_Resistance_Color       C'255,53,17'             C'179,0,0'
Pivot_Support_Color          C'64,64,255'             C'90,75,173' 
Pivot_Mid_Color              Olive                    C'85,85,0' 

Range Lines Section-
RDH_Color                    C'255,90,62'             Red
RDL_Color                    C'106,106,255'           C'0,115,230'
RWH_Color                    C'255,90,62'             Red
RWL_Color                    C'106,106,255'           C'0,115,230' 

Separators Section-
Separators_Color             BlueViolet               C'85,85,85'

vLines Section-
vLabels_Color                MidnightBlue             Olive  
vLines_Color                 Silver                   C'60,60,0'                     
Location_MktClosed           Black                    C'38,166,38'

Clock Section-
Location_MktClosed           C'80,80,80'              C'120,120,120'
Clock_MktClosed              C'80,80,80'              C'120,120,120'
Location_MktOpen             C'0,149,34'              C'36,157,36' 
Clock_MktOpen                C'133,84,35'             C'174,111,47'                                                                                                                                                             
---------------------------------------------------------------------------------------------*/


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+ 
#property indicator_chart_window

#include <stdlib.mqh>
//See function documentation at http://msdn.microsoft.com/en-us/library/ms725473(VS.85).aspx
#import "kernel32.dll"
void GetLocalTime(int& LocalTimeArray[]);
void GetSystemTime(int& systemTimeArray[]);
int  GetTimeZoneInformation(int& LocalTZInfoArray[]);
bool SystemTimeToTzSpecificLocalTime(int& targetTZinfoArray[], 
     int& systemTimeArray[], int& targetTimeArray[]);
#import

//Global External Inputs-----------------------------------------------------------------------
extern bool   Indicator_On                    = true;
extern string Preface_________________        = "(Read User Notes)   Use Clock w/live";
extern string Instruction                     = "feed to get input.  Apply w/o live feed.";
extern int    Broker_GMT_Offset               = 0;  
extern bool   Apply_it_now                    = false;
extern string End_Preface_____________        = "";
 
extern string _                               = "";
extern string Part_1                          = "Indicator Master Controls:";
extern int    Main_Panel_Max_TF               = 43200;
extern bool   Bid_Line_On                     = true;
extern bool   Levels_On                       = true;
extern int    Levels_Max_TF                   = 60;
extern bool   Daily_Main_Pivot_On             = true;
extern int    Daily_Main_Pivot_Max_TF         = 60;  
extern bool   Daily_SR_Pivot_Lines_On         = true;
extern int    Daily_SR_Pivots_Max_TF          = 60;  
extern bool   Daily_Pivots_Use_Fib_Formula    = false;
extern bool   TzPivots_On                     = true;
extern int    TzPivots_GMT_Offset             = 3;
extern bool   Day_Range_Lines_On              = true;
extern int    Day_Range_Lines_Max_TF          = 60;
extern bool   Week_Range_Lines_On             = true;
extern int    Week_Range_Lines_Min_TF         = 15; 
extern int    Week_Range_Lines_Max_TF         = 10080;
extern bool   Separators_On                   = false;
extern int    Prior_Days_To_Show_Separators   = 0;
extern int    Separators_Max_TF               = 240;
extern bool   vLines_On                       = true;
extern int    vLines_Prior_Sessions_To_Show   = 10;
extern int    vLines_Max_TF                   = 60;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";
 
extern string __                              = "";
extern string Part_2                          = "Panel Settings:";
extern bool   Use_For_Forex                   = true;
extern bool   Display_5_Digits_Forex          = false;
extern int    Digits_To_Show_In_Spread        = 2;
extern color  Panel_Background_Color          = C'238,238,255';
extern color  Symbol_And_TF_Color             = Black; 
extern color  Spread_Color                    = Black;
extern int    Days_For_Range_Day_Average      = 15; 
extern int    Weeks_For_Range_Week_Average    = 12;   
extern color  Range_Color                     = Black;
extern color  Candle_Time_Color               = Black;
extern color  PriceLabel_UP_Color             = ForestGreen;
extern color  PriceLabel_DOWN_Color           = Crimson;
extern color  sonicS                          = Blue;
extern color  sonicO                          = C'111,111,255';
extern color  sonicN                          = C'170,170,255';
extern color  sonicI                          = C'252,154,156';  
extern color  sonicC                          = C'239,61,105';    
extern color  sonicR                          = C'240,21,77';   

extern string ___                             = "";
extern string Part_3                          = "Bid Ray Line Settings:";
extern color  Bid_Ray_Color                   = Black;
extern int    Bid_Ray_LineStyle_01234         = 0;  
extern int    Bid_Ray_Thickness               = 1;
extern int    Bid_Ray_TFs_Past_Last_Candle    = 3;
extern color  Bid_Dot_Up_Color                = Lime;
extern color  Bid_Dot_Dn_Color                = OrangeRed;     
extern int    Bid_Dot_Size                    = 12;

extern string ____                            = "";
extern string Part_4                          = "Horizontal Lines Position Settings:";
extern string note_4.1                        = "0 = Full screen";
extern string note_4.2                        = "1 = left thru current session";
extern string note_4.3                        = "2 = Current session only";
extern string note_4.4                        = "3 = Current session to right";
extern string note_4.5                        = "4 = Last candle to right (+Shift_Right)";
extern int    Level_Whole_Display_Number      = 0;
extern int    Level_Half_Display_Number       = 0;
extern int    Level_Quarter_Display_Number    = 0;
extern int    Pivot_Main_Display_Number       = 4;
extern int    Pivot_SR_Display_Number         = 4;
extern int    Range_Day_Display_Number        = 4;
extern int    Range_Week_Display_Number       = 4;
extern string note_4.6                        = "Shift_Line_Right Settings:";
extern string note_4.7                        = "0 = shift 0 right";
extern string note_4.8                        = "1 = shift + right";
extern string note_4.9                        = "2 = shift ++ right";
extern string note_4.10                       = "3 = shift +++ right";
extern string note_4.11                       = "4 = shift ++++ right";
extern string note_4.12                       = "5 = shift max right";
extern int    Level_Whole_Shift_Line_Right    = 0;
extern int    Level_Half_Shift_Line_Right     = 0;
extern int    Level_Quarter_Shift_Line_Right  = 0;
extern int    Pivot_Main_Shift_Line_Right     = 5;
extern int    Pivot_SR_Shift_Line_Right       = 5;
extern int    Range_Day_Shift_Line_Right      = 5;
extern int    Range_Week_Shift_Line_Right     = 4;
extern string note_4.13                       = "Wide monitors may require";
extern string note_4.14                       = "WindowsBarsPerChart be increased";
extern string note_4.15                       = "for each chart zoom setting:";
extern int    Zoom_Out_2_Bars                 = 1200;
extern int    Zoom_Out_1_Bars                 = 500;
extern int    Zoom_Std_Bars                   = 200;
extern int    Zoom_In_1_Bars                  = 100;
extern int    Zoom_In_2_Bars                  = 60;
extern bool   Lines_Subodinated               = true;

extern string _____                           = "";
extern string Part_5                          = "Horizontal Lines Labels Settings:";
extern bool   Levels_LVL_Labels_Shown         = false;
extern bool   Pivot_DPV_Label_Shown           = true;
extern bool   Pivot_SR_Labels_Shown           = true;
extern bool   Pivot_Mid_Labels_Shown          = true;
extern bool   Range_Day_HL_Labels_Shown       = true;
extern bool   Range_Week_HL_Labels_Shown      = true;
extern bool   Line_Labels_Prices              = false;
extern bool   Line_Labels_Subodinated         = true;
extern string Line_Labels_FontStyle           = "Verdana"; 
extern int    Line_Labels_FontSize            = 8;  
extern color  Line_Labels_Color               = MidnightBlue;

extern string ______                          = "";
extern string Part_6                          = "Individual Levels Settings:"; 
extern int    Override_Line_Sets_Displayed    = 0;
extern bool   Level_Half_Lines_On             = true;
extern bool   Level_Quarter_Lines_On          = true;
extern color  Level_Whole_Line_Color          = C'235,235,252';   
extern int    Line1_Style_01234               = 0;
extern int    Line1_SolidThickness            = 3;
extern color  Level_Half_Line_Color           = C'210,251,210'; 
extern int    Line2_Style_01234               = 0;
extern int    Line2_SolidThickness            = 2;
extern color  Level_Quarter_Line_Color        = C'242,238,227';   
extern int    Line3_Style_01234               = 0;
extern int    Line3_SolidThickness            = 2;

extern string _______                         = "";
extern string Part_7                          = "Individual Pivots Settings:"; 
extern color  DPV1_Background_Color           = Yellow;
extern int    DPV1_LineStyle_01234            = 0;
extern int    DPV1_SolidLineThickness         = 3; 
extern color  DPV2_Top_Color                  = Gray;
extern int    DPV2_LineStyle_01234            = 0;
extern int    DPV2_SolidLineThickness         = 1; 
extern color  Pivot_Resistance_Color          = C'255,53,17'; 
extern int    R_LineStyle_01234               = 0;
extern int    R_SolidLineThickness            = 1; 
extern color  Pivot_Support_Color             = C'64,64,255';
extern int    S_LineStyle_01234               = 0;
extern int    S_SolidLineThickness            = 1;
extern color  MidPivots_Color                 = Olive;
extern int    mP_LineStyle_01234              = 0;
extern int    mP_SolidLineThickness           = 1;
extern bool   MidPivots_On                    = true; 
 
extern string ________                        = "";
extern string Part_8                          = "Individual Range Lines Settings:";
extern color  RDH_Color                       = C'255,90,62';
extern color  RDL_Color                       = C'106,106,255';
extern int    D_LineStyle_01234               = 0;    
extern int    D_SolidLineThickness            = 2;
extern color  RWH_Color                       = C'255,90,62';
extern color  RWL_Color                       = C'106,106,255';
extern int    W_LineStyle_01234               = 0;    
extern int    W_SolidLineThickness            = 3;

extern string _________                       = "";
extern string Part_9                          = "Separators Selections:";
extern color  Separators_Color                = BlueViolet;  
extern int    Separators_Style_01234          = 2;    
extern int    Separators_Solid_Thickness      = 1;
extern bool   Separators_Thru_SubWindows      = true;

extern string __________                      = "";
extern string Part_10                         = "vLines Selections:";
extern bool   Show_vLabels                    = true;
extern color  vLabels_Color                   = MidnightBlue; 
extern string vLabels_FontStyle               = "Verdana";
extern int    vLabels_FontSize                = 7;
extern bool   vLabels_Chart_Top               = true;  //true; //false;
extern int    vLabels_Dist_To_Border          = 1;
extern string Label_London_Open               = "      Lo";
extern string Label_NewYork_Open              = "      Ny";
extern string Label_USStock_Open              = "      Ex";
extern string Label_London_Close              = "      Lc";
extern color  vLines_Color                    = Silver; 
extern int    vLines_Style_01234              = 2;    
extern int    vLines_Solid_Thickness          = 1;
extern bool   vLines_Thru_SubWindows          = true;

extern string ___________                     = "";
extern string Part_11                         = "Clock Display Controls:"; 
extern color  Location_MktClosed              = C'80,80,80';        
extern color  Clock_MktClosed                 = C'80,80,80';  
extern color  Location_MktOpen                = C'0,149,34';     
extern color  Clock_MktOpen                   = C'133,84,35';  
extern bool   Show_AMPM_Time                  = true;
extern int    Clock_Min_TF                    = 1;
extern int    Clock_Max_TF                    = 43200;
extern string Note_11.1                       = "Choosing Locations to Display:";
extern string Note_11.2                       = "Select 7 maximum from list,";
extern string Note_11.3                       = "0 = display off.";
extern string Note_11.4                       = "1 = Display on.";
extern int    Broker                          = 0;
extern int    Local                           = 0; //Your computer's time.
extern int    Auckland                        = 0;
extern int    Sydney                          = 1;
extern int    Tokyo                           = 1;
extern int    HongKong                        = 1;
extern int    Jakarta                         = 0;
extern int    India                           = 0;
extern int    Dubai                           = 0;
extern int    Moscow                          = 0;
extern int    Israel                          = 0;
extern int    Helsinki                        = 1;
extern int    Frankfurt                       = 1;
extern int 	  London                          = 1; //DST London=GMT+1, NonDST London=GMT
extern int    GMT                             = 0; //No change btwn DST/summer & NonDST/winter
extern int    Brazil                          = 0;
extern int    NewYork                         = 1;
extern int    Central                         = 0;
extern int    Mexico                          = 0;
extern int    Mountain                        = 0;
extern int    Pacific                         = 0;


//Buffers, Constants and Variables-------------------------------------------------------------
bool       FLAG_deinit;

//Deinit
int        obj_total,k;
string     name;

//Panel
color      Static_Price_Color, Static_Bid_Color, Bid_Dot_Color, Static_Bid_Dot_Color;
int        fxDigits,modifier,Color,Factor,G,h,i, m,s,t,T1,T2;
double     Poin, PoinFx, Old_Price, fxPrice, level, Spread, ARg, Range, TodayRange;  
string     item1  = "[CP Box] Box 2";  
string     item2  = "[CP Box] Box 3"; 
string     item7  = "[CP Data] ID&TF";
string     item8  = "[CP Data] Spread"; 
string     item9  = "[CP Data] Range";     
string     item10 = "[CP Data] Time"; 
string     item11 = "[CP Data] Price";
string     item12 = "[CP Logo] Logo1 S"; 
string     item13 = "[CP Logo] Logo1 o"; 
string     item14 = "[CP Logo] Logo1 n";     
string     item15 = "[CP Logo] Logo1 i"; 
string     item16 = "[CP Logo] Logo1 c";
string     item17 = "[CP Logo] Logo1 R"; 
string     item18 = "[CP Art] Bid Line";
string     item19 = "[CP Art] Bid Dot";
string     ID,sub1,sub2,Price,C,tab,tab2,timeleft; 

//Levels
color      linecolor;
double     linelevel, linestyle, linewidth, BL;
int        j, Z;
int        Line1_Color, Line2_Color, Line3_Color; 
int        u1=00, u2=50, u3=25, u4=75;        
int        v1, ssp, ssp1, NumberOfRanges;
string     linename;

//Range Day
int        ii,iii,x,xx,r,Hbarshift,Lbarshift,rdc,rwc;
double     HiToday,LoToday;
double     RangeAvg, RangeHigh, RangeLow, RH, RL;

//Range Week
int        wHbarshift, wLbarshift;
datetime   Weekshift;
double     HiWeek, LoWeek;
double     wRangeAvg, wRangeHigh, wRangeLow, wRH, wRL;

//Pivots
double     p,d,q,r1,r2,r3,r4,r5,s1,s2,s3,s4,s5;
int        idxfirstbaryesterday, idxlastbaryesterday, idxfirstbartoday;     
double     today_high, today_low, yesterday_high, yesterday_open, 
           yesterday_low, yesterday_close;                         
string     prefix;

//Lines
int        a,b,c,R2,T,T4;
int        Zoom_Out_2, Zoom_Out_1, Zoom_Std, Zoom_In_1, Zoom_In_2, Zoom_In_3;   
string     line, dt;
datetime   startline, stopline;

//Labels
string     linelabel, spc;
datetime   startlabel, startrangeday; 

//vLines
int        bps, MF,time3;
datetime   shift, T3;
double     top, bottom, scale,YadjustTop,YadjustBot,vLabels_Dist_Above_Border,time2,time4;
string     vline,vlabel;
string     Label_Day_Separator = "day";       

//Get GMT Offsets in Support of TzPivots and Market vLines                    
int        GMT_Ref_Time, London_GMT_Offset, NYC_GMT_Offset, GMT_At_Lo, GMT_At_NYo, GMT_At_Lc; 

//Import Time Zone Data in Support of Clock, TzPivots and Market vLines
bool       FLAG_match, FLAG2_match, FLAG_static_broker;
int        ClockFontSize, TimezoneFontSize, LineSpacing, z, time, place, offset;              
int        AucklandTZInfoArray[43],SydneyTZInfoArray[43],TokyoTZInfoArray[43],
           ChinaTZInfoArray[43],JakartaTZInfoArray[43],IndiaTZInfoArray[43],
           DubaiTZInfoArray[43],MoscowTZInfoArray[43],IsraelTZInfoArray[43],
           HelsinkiTZInfoArray[43],BerlinTZInfoArray[43],LondonTZInfoArray[43],
           BrazilTZInfoArray[43],NewYorkTZInfoArray[43],CentralTZInfoArray[43],
           MexicoTZInfoArray[43],MountainTZInfoArray[43],PacificTZInfoArray[43],
           LocalTZInfoArray[43];          
int        LocalTimeArray[4],systemTimeArray[4],AucklandTimeArray[4],SydneyTimeArray[4],
           TokyoTimeArray[4],ChinaTimeArray[4],JakartaTimeArray[4],IndiaTimeArray[4],
           DubaiTimeArray[4],MoscowTimeArray[4],IsraelTimeArray[4],HelsinkiTimeArray[4],
           BerlinTimeArray[4],LondonTimeArray[4],BrazilTimeArray[4],NewYorkTimeArray[4],
           CentralTimeArray[4],MexicoTimeArray[4],MountainTimeArray[4],PacificTimeArray[4];         
datetime   Local_Time,BrokerDT,AucklandDT,SydneyDT,TokyoDT,ChinaDT,JakartaDT,IndiaDT,
           DubaiDT,MoscowDT,IsraelDT,HelsinkiDT,BerlinDT,LondonDT,UTC,BrazilDT,NewYorkDT,
           CentralDT,MexicoDT,MountainDT,PacificDT,StaticBrokerDT;
string     Locals,Brokers,Aucklands,Sydneys,Tokyos,Chinas,Jakartas,Indias,Dubais,
           Moscows,Israels,Helsinkis,Berlins,Londons,UTCs,Brazils,NewYorks,Centrals,
           Mexicos,Mountains,Pacifics;           
string     Brokerp, Localp, Aucklandp, Sydneyp, Tokyop, Chinap, Jakartap,  
           Indiap, Dubaip, Moscowp, Israelp, Helsinkip, Berlinp, Londonp, UTCp, 
           Brazilp, NYp, Centralp, Mexicop, Mountainp, Pacificp, FontName; 
string     timeStr,LocalTimeS,sMonth,sDay,sHour,sMin,sSec;
int        hour,nYear,nMonth,nDay,nHour,nMin,nSec,nMilliSec;  
                      
//Clock - Normal hrs 8-17 (9 hrs), Sydney 9-18 (9 hrs) and Tokyo 9-18 (9 hrs)   
int        LocalOpenHour        =  8;    
int        LocalCloseHour       = 17;
int        SydneyLocalOpenHour  =  9;       
int        SydneyLocalCloseHour = 18;
int        TokyoLocalOpenHour   =  9;        
int        TokyoLocalCloseHour  = 18; 
                   
//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+      
int init()
  {
  //With the first DLL call below, the program will exit (and stop) automatically after one alert. 
  if(!IsDllsAllowed() ) 
    {
    Alert("SonicR Control Panel: Allow DLL Imports"); return(false);  
    }
    
  //If one, or both, Range Line settings are invalid, exit------------------------------------
  if(Level_Whole_Display_Number <0 || Level_Whole_Display_Number >4 ||
    Level_Half_Display_Number <0 || Level_Half_Display_Number >4 ||
    Level_Quarter_Display_Number <0 || Level_Quarter_Display_Number >4 ||
    Pivot_Main_Display_Number <0 || Pivot_Main_Display_Number >4 ||        
    Pivot_SR_Display_Number <0 || Pivot_SR_Display_Number >4 ||
    Range_Day_Display_Number <0 || Range_Day_Display_Number >4 ||
    Range_Week_Display_Number <0 || Range_Week_Display_Number >4)      
    {
    Alert("SonicR Control Panel: Range/SR Line Number invalid"); return(false);
    }   
  
  //Signal State of Non-deinitialization 
  FLAG_deinit = false;
  
  //Panel - setting background width   
  G= 124; if(Show_AMPM_Time) {G= 142;}
             
  //Panel - determine display of Market Price
  if(Use_For_Forex)
    {
    sub1=StringSubstr(Symbol(), 3, 3);
    sub2=StringSubstr(Symbol(), 0, 2);
    if(Display_5_Digits_Forex) 
      {
      fxDigits = 5;
      if (sub1 == "JPY") {fxDigits = 3;} 
      if (sub2 == "XA")  {fxDigits = 2;}
      }        
    else 
      {
      fxDigits = 4;
      if (sub1 == "JPY") {fxDigits = 2;}
      if (sub2 == "XA")  {fxDigits = 2;} 
      }         
    } 
  if (Digits == 5 || Digits == 3) {Factor = 10;} 
  else {Factor = 1;} //cater for 5 digits 
    
  //Panel and Levels - Check for unconventional Point digits number 
  //for display of market price
  if (Point == 0.00001) {Poin = 0.0001;} //5 digits     
  else if (Point == 0.001) {Poin = 0.01;} //3 digits
  else {Poin = Point;} //Normal 4 digits

  if (StringSubstr(Symbol(),0,6) == "XAUUSD") {Poin = 0.1;}  
  if (StringSubstr(Symbol(),0,6) == "USDMXN") {Poin = 0.001;}  
  if (StringSubstr(Symbol(),0,6) == "USDCZK") {Poin = 0.001;}  

  //Panel - Make Labels
  MakeLabel( item1, 0, 0);
  MakeLabel( item2, G, 0);
  MakeLabel( item7, 0,2);
  MakeLabel( item8, 28,18);
  MakeLabel( item9, 28,31);
  MakeLabel( item10, 28,44);
  MakeLabel( item11, 5,55);  
  MakeLabel( item12, 143,2);      
  MakeLabel( item13, 144,15);//14  
  MakeLabel( item14, 144,26);//25 
  MakeLabel( item15, 146,40);//39
  MakeLabel( item16, 144,50);
  MakeLabel( item17, 142,62);  
     
  //Levels - set ranges to user specified #, or use a preset number
  if((Levels_On) && (Override_Line_Sets_Displayed >= 1)) 
    {
    NumberOfRanges = Override_Line_Sets_Displayed-1;
    }   
  else if(Period() == 1) {NumberOfRanges = 4;}
  else if(Period() == 5) {NumberOfRanges = 7;}
  else if(Period() == 15) {NumberOfRanges = 15;}
  else if(Period() == 30) {NumberOfRanges = 15;}   
  else if(Period() == 60) {NumberOfRanges = 20;}  
  else if(Period() == 240) {NumberOfRanges = 30;}  
  else if(Period() == 1440) {NumberOfRanges = 30;} 
  else if(Period() >= 10080) {NumberOfRanges = -1;} 

  //Clock - Import Time Zone Data & Make Clock Labels
  FLAG_match  = false; 
  FLAG2_match = false;       
  StaticBrokerDT = TimeCurrent();           
  GetAllTimeZoneInfo(AucklandTZInfoArray, SydneyTZInfoArray, TokyoTZInfoArray, 
    ChinaTZInfoArray, JakartaTZInfoArray, IndiaTZInfoArray, DubaiTZInfoArray, 
    MoscowTZInfoArray, IsraelTZInfoArray, HelsinkiTZInfoArray, BerlinTZInfoArray, 
    LondonTZInfoArray, BrazilTZInfoArray, NewYorkTZInfoArray, CentralTZInfoArray, 
    MexicoTZInfoArray, MountainTZInfoArray, PacificTZInfoArray); 
    
    LineSpacing = 11;//12 
    top   = 3;   //vertical start text
    offset= 0;   //vertical "LineSpacing" additions to "top" per line of text
    time  = 180; //horizontal start time (same for 24hr and am/pm times), 140
    place = 215; //horizontal start location (if using 24hr format), 177
    if(Show_AMPM_Time) //shift to right to accomodate extra time characters
      {                
      place= 232; //shift location right, 196
      }   
  	 //Begin world timezones, listed IN ORDER!
    if(Broker == 1)    	  	
      {
      MakeLabel( "[CP Time] Brokert", time, top+offset );
      MakeLabel( "[CP Time] Brokerl", place, top+offset );
     	offset+=LineSpacing;
      } 
    if(Auckland == 1)         	
      {                  	     	       	        
   	MakeLabel( "[CP Time] Aucklandt", time, top+offset );         
   	MakeLabel( "[CP Time] Aucklandl", place, top+offset );
   	offset+=LineSpacing;    
      }
    if(Sydney == 1)      
      {               
   	MakeLabel( "[CP Time] Sydneyt", time, top+offset );
   	MakeLabel( "[CP Time] Sydneyl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Tokyo == 1)
      {
   	MakeLabel( "[CP Time] Tokyot", time, top+offset );
   	MakeLabel( "[CP Time] Tokyol", place, top+offset );
   	offset+=LineSpacing;
      }
    if(HongKong == 1)       
      {
   	MakeLabel( "[CP Time] Chinat", time, top+offset );
   	MakeLabel( "[CP Time] Chinal", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Jakarta == 1)        
      {
   	MakeLabel( "[CP Time] Jakartat", time, top+offset );
   	MakeLabel( "[CP Time] Jakartal", place, top+offset );
   	offset+=LineSpacing;
      }
    if(India == 1)      
      {
   	MakeLabel( "[CP Time] Indiat", time, top+offset );
   	MakeLabel( "[CP Time] Indial", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Dubai == 1)        
      {
   	MakeLabel( "[CP Time] Dubait", time, top+offset );
   	MakeLabel( "[CP Time] Dubail", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Moscow == 1)       
      {
   	MakeLabel( "[CP Time] Moscowt", time, top+offset );
   	MakeLabel( "[CP Time] Moscowl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Israel == 1)       
      {
   	MakeLabel( "[CP Time] Israelt", time, top+offset );
   	MakeLabel( "[CP Time] Israell", place, top+offset );
   	offset+=LineSpacing;
      }      
    if(Helsinki == 1)        
      {                           	       
   	MakeLabel( "[CP Time] Helsinkit", time, top+offset );
   	MakeLabel( "[CP Time] Helsinkil", place, top+offset );
   	offset+=LineSpacing; 	
      }      
    if(Frankfurt == 1)        
      {
   	MakeLabel( "[CP Time] Berlint", time, top+offset );
   	MakeLabel( "[CP Time] Berlinl", place, top+offset );
   	offset+=LineSpacing;
      }      
    if(London == 1)
      {         
	   MakeLabel( "[CP Time] Londont", time, top+offset );
   	MakeLabel( "[CP Time] Londonl", place, top+offset );
   	offset+=LineSpacing;
      }      
    if(GMT == 1)        
      {
   	MakeLabel( "[CP Time] utct", time, top+offset );
   	MakeLabel( "[CP Time] utcl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Brazil == 1)      
      {
   	MakeLabel( "[CP Time] Brazilt", time, top+offset );
   	MakeLabel( "[CP Time] Brazill", place, top+offset );
   	offset+=LineSpacing;
      }
    if(NewYork == 1)       
      { 
	   MakeLabel( "[CP Time] NewYorkt", time, top+offset );
   	MakeLabel( "[CP Time] NewYorkl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Central == 1)        
      {
   	MakeLabel( "[CP Time] Centralt", time, top+offset );
   	MakeLabel( "[CP Time] Centrall", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Mexico == 1)        
      { 
   	MakeLabel( "[CP Time] Mexicot", time, top+offset );
   	MakeLabel( "[CP Time] Mexicol", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Mountain == 1)        
      {
   	MakeLabel( "[CP Time] Mountaint", time, top+offset );
   	MakeLabel( "[CP Time] Mountainl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Pacific == 1)        
      {
	   MakeLabel( "[CP Time] Pacifict", time, top+offset );
   	MakeLabel( "[CP Time] Pacificl", place, top+offset );
   	offset+=LineSpacing;
      }
    if(Local == 1)        
      {     
      MakeLabel( "[CP Time] Localt", time, top+offset );
      MakeLabel( "[CP Time] Locall", place, top+offset );
      offset+=LineSpacing;
      }                            

  return(0);
  }

//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+       
int deinit()
  {   
  obj_total= ObjectsTotal();  
  for(k= obj_total; k>=0; k--)
    {
    name= ObjectName(k);
    if (StringSubstr(name,0,3)=="[CP"){ObjectDelete(name);}
    }
  //Comment ("");
  return(0);
  }

//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+         
int start()
  {
  //If Indicator is "Off" deinitialize only once, not every tick-------------------------------  
  if(!Indicator_On) 
    {
    if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
    return(0);
    }
            
  //If indicator is "ON", show deinit() is false and run---------------------------------------
  //deinit only [CP Art} objects, deinit() remains false, and run
  FLAG_deinit = false;
  obj_total= ObjectsTotal();  
  for(k= obj_total; k>=0; k--)
    {
    name= ObjectName(k); 
    if (StringSubstr(name,0,8)=="[CP Art]"){ObjectDelete(name);} 
    }   

  //Panel and Range lines - Define today's bar/data--------------------------------------------                   
  HiToday    = iHigh(NULL,PERIOD_D1,0);
  LoToday	 = iLow (NULL,PERIOD_D1,0);	
  Hbarshift  = iHighest(NULL,0,MODE_HIGH,1440/Period(),0); 
  Lbarshift  = iLowest(NULL,0,MODE_LOW,1440/Period(),0);     
  TodayRange = ((HiToday - LoToday)/Point)/Factor;

  //Up/down color selection when bid price changes--------------------------------------------- 
  //Color is to remain as-is until price changes direction by a whole pip, or by a fractional
  //pip if 5 digits is displayed.  We don't want a 3rd color on ticks without a price change.
  if(Use_For_Forex)
    {  
    if(Display_5_Digits_Forex) {fxPrice = Bid;}        
    else 
      {
      PoinFx = Poin;         
      if (sub2 == "XA") {PoinFx = 0.01;}
      fxPrice = MathFloor(Bid/PoinFx)*PoinFx;
      }          
    Price=DoubleToStr(fxPrice, fxDigits);
    }      
  else 
    {
    fxPrice = Bid;
    Price=DoubleToStr(Bid, Digits);
    } 
                 
  if(fxPrice > Old_Price) 
    {
    Color = PriceLabel_UP_Color;
    Static_Price_Color = Color;     
    Bid_Dot_Color = Bid_Dot_Up_Color;       
    Static_Bid_Dot_Color = Bid_Dot_Up_Color;   
    }
  else if (fxPrice < Old_Price) 
    {      
    Color = PriceLabel_DOWN_Color; 
    Static_Price_Color = Color;  
    Bid_Dot_Color = Bid_Dot_Dn_Color;  
    Static_Bid_Dot_Color = Bid_Dot_Dn_Color;  
    }   
  else //if (fxPrice == Old_Price)  
    {
    Color = Static_Price_Color;       
    Bid_Dot_Color = Static_Bid_Dot_Color;
    }
  Old_Price = fxPrice;
      
  //Panel--------------------------------------------------------------------------------------    
  //Background boxes      
  ObjectSet(item1, OBJPROP_BACK, false); 
  ObjectSetText(item1, "gg", 64, "Webdings", Panel_Background_Color);         
  //-----         
  ObjectSet(item2, OBJPROP_BACK, false); 
  ObjectSetText(item2, "gg", 64, "Webdings", Panel_Background_Color);  
     
  //Symbol and Time Frame
  ID = StringSubstr(Symbol(), 0, 6);        	 	         
  C = " "; if (Period()== 1)     C =C +  "    "+ID+"   M1";
     else {if (Period()== 5)     C =C +  "    "+ID+"   M5";
     else {if (Period()== 15)    C =C +  "   " +ID+"   M15";
     else {if (Period()== 30)    C =C +  "   " +ID+"   M30";
     else {if (Period()== 60)    C =C +  "    " +ID+"   H1";
     else {if (Period()== 240)   C =C +  "    " +ID+"   H4";
     else {if (Period()== 1440)  C =C +  "   "  +ID+"   Daily";
     else {if (Period()== 10080) C =C +  " "    +ID+"   Weekly";
     else {if (Period()== 43200) C =C +  ""     +ID+"   Monthly"; }}}}}}}} 
  ObjectSetText(item7, C, 11, "Arial Bold", Symbol_And_TF_Color); 
                          
  //Spread
  Spread = MarketInfo(Symbol(), MODE_SPREAD);
  Spread = Spread/Factor;                                        
  ObjectSetText(item8, "Spread   "+
     DoubleToStr(Spread,Digits_To_Show_In_Spread), 8, "Arial", Spread_Color);  
   
  //Range 
  Ranges(Days_For_Range_Day_Average);
  Range = ((ARg/Days_For_Range_Day_Average)/Point)/Factor;   
  ObjectSetText(item9, "Range    "+DoubleToStr(Range,0)+
    ", "+DoubleToStr(TodayRange,0), 8, "Arial", Range_Color);                
                           
  //Time To Next Candle	   
  if(Period()>1440) {timeleft = " (OFF)";}
  else {timeleft =TimeToStr(Time[0]+Period()*60-TimeCurrent(),TIME_MINUTES|TIME_SECONDS);}                  
  ObjectSetText(item10, "Candle   "+timeleft, 8, "Arial", Candle_Time_Color);          
      
  //Market Price  
  //Add leading spaces to center price labels
  if(Display_5_Digits_Forex) 
    {                 
	 tab= " ";
	      if(Symbol()== "USDMXN"){tab= "";}	   
    else if(Symbol()== "XAUUSD"){tab= " ";} 	 	     	 
    else if(Symbol()== "USDJPY"){tab= "  ";}
    else if(Symbol()== "CHFJPY"){tab= "  ";}
    else if(Symbol()== "AUDJPY"){tab= "  ";}
    else if(Symbol()== "CADJPY"){tab= "  ";}
    else if(Symbol()== "XAGUSD"){tab= "  ";} 
    }    
  else //if(!Display_5_Digits_Forex) 
    {
    tab= "  "; 
	      if(Symbol()== "USDMXN"){tab= " ";}	   
    else if(Symbol()== "XAUUSD"){tab= " ";} 	 	     	 
    else if(Symbol()== "USDJPY"){tab= "   ";}
    else if(Symbol()== "CHFJPY"){tab= "   ";}
    else if(Symbol()== "AUDJPY"){tab= "   ";}
    else if(Symbol()== "CADJPY"){tab= "   ";}
    else if(Symbol()== "XAGUSD"){tab= "   ";}
    }       
  ObjectSetText(item11, tab+Price+tab2, 18, "Verdana Bold",Color); 

  //Create SonicR Logo            
  ObjectSetText(item12, "S", 12, "Arial Bold",sonicS);      
  ObjectSetText(item13, "o", 10, "Arial Bold",sonicO);      
  ObjectSetText(item14, "n", 10, "Arial Bold",sonicN);   
  ObjectSetText(item15, "i", 10, "Arial Bold",sonicI);           
  ObjectSetText(item16, "c", 10, "Arial Bold",sonicC);       
  ObjectSetText(item17, "R", 13, "Verdana Bold",sonicR);  
 
  //Clock--------------------------------------------------------------------------------------
    {
    GetSystemTime(systemTimeArray);
    UTC = TimeArrayToTime(systemTimeArray);         
    BrokerDT = TimeCurrent();      
    SystemTimeToTzSpecificLocalTime(AucklandTZInfoArray, systemTimeArray, AucklandTimeArray);
    AucklandDT = TimeArrayToTime(AucklandTimeArray);      
    SystemTimeToTzSpecificLocalTime(SydneyTZInfoArray, systemTimeArray, SydneyTimeArray);
    SydneyDT   = TimeArrayToTime(SydneyTimeArray);   
    SystemTimeToTzSpecificLocalTime(TokyoTZInfoArray, systemTimeArray, TokyoTimeArray);
    TokyoDT    = TimeArrayToTime(TokyoTimeArray);      
    SystemTimeToTzSpecificLocalTime(ChinaTZInfoArray, systemTimeArray, ChinaTimeArray);
    ChinaDT    = TimeArrayToTime(ChinaTimeArray);      
    SystemTimeToTzSpecificLocalTime(JakartaTZInfoArray, systemTimeArray, JakartaTimeArray);
    JakartaDT  = TimeArrayToTime(JakartaTimeArray);      
    SystemTimeToTzSpecificLocalTime(IndiaTZInfoArray, systemTimeArray, IndiaTimeArray);
    IndiaDT    = TimeArrayToTime(IndiaTimeArray);      
    SystemTimeToTzSpecificLocalTime(DubaiTZInfoArray, systemTimeArray, DubaiTimeArray);
    DubaiDT    = TimeArrayToTime(DubaiTimeArray);      
    SystemTimeToTzSpecificLocalTime(MoscowTZInfoArray, systemTimeArray, MoscowTimeArray);
    MoscowDT   = TimeArrayToTime(MoscowTimeArray);      
    SystemTimeToTzSpecificLocalTime(IsraelTZInfoArray, systemTimeArray, IsraelTimeArray);
    IsraelDT   = TimeArrayToTime(IsraelTimeArray);      
    SystemTimeToTzSpecificLocalTime(HelsinkiTZInfoArray, systemTimeArray, HelsinkiTimeArray);
    HelsinkiDT = TimeArrayToTime(HelsinkiTimeArray);        
    SystemTimeToTzSpecificLocalTime(BerlinTZInfoArray, systemTimeArray, BerlinTimeArray);
    BerlinDT   = TimeArrayToTime(BerlinTimeArray);      
    SystemTimeToTzSpecificLocalTime(LondonTZInfoArray, systemTimeArray, LondonTimeArray);
    LondonDT   = TimeArrayToTime(LondonTimeArray);     
    SystemTimeToTzSpecificLocalTime(BrazilTZInfoArray, systemTimeArray, BrazilTimeArray);
    BrazilDT   = TimeArrayToTime(BrazilTimeArray);   
    SystemTimeToTzSpecificLocalTime(NewYorkTZInfoArray, systemTimeArray, NewYorkTimeArray);
    NewYorkDT  = TimeArrayToTime(NewYorkTimeArray);   
    SystemTimeToTzSpecificLocalTime(CentralTZInfoArray, systemTimeArray, CentralTimeArray);
    CentralDT  = TimeArrayToTime(CentralTimeArray);      
    SystemTimeToTzSpecificLocalTime(MexicoTZInfoArray, systemTimeArray, MexicoTimeArray);
    MexicoDT   = TimeArrayToTime(MexicoTimeArray);      
    SystemTimeToTzSpecificLocalTime(MountainTZInfoArray, systemTimeArray, MountainTimeArray);
    MountainDT = TimeArrayToTime(MountainTimeArray);      
    SystemTimeToTzSpecificLocalTime(PacificTZInfoArray, systemTimeArray, PacificTimeArray);
    PacificDT  = TimeArrayToTime(PacificTimeArray);      
    GetLocalTime(LocalTimeArray);
    Local_Time = TimeArrayToTime(LocalTimeArray);   
   
    Brokers   = TimeToString( TimeCurrent() );   
    Aucklands = TimeToString( AucklandDT  );
    Sydneys   = TimeToString( SydneyDT  );
    Tokyos    = TimeToString( TokyoDT  );
    Chinas    = TimeToString( ChinaDT  );
    Jakartas  = TimeToString( JakartaDT  );
    Indias    = TimeToString( IndiaDT  );
    Dubais    = TimeToString( DubaiDT  );
    Moscows   = TimeToString( MoscowDT  );
    Israels   = TimeToString( IsraelDT  );
    Helsinkis = TimeToString( HelsinkiDT  );
    Berlins   = TimeToString( BerlinDT  );
    Londons   = TimeToString( LondonDT  );
    UTCs      = TimeToString( UTC );
    Brazils   = TimeToString( BrazilDT  );
    NewYorks  = TimeToString( NewYorkDT  );
    Centrals  = TimeToString( CentralDT  );
    Mexicos   = TimeToString( MexicoDT  );
    Mountains = TimeToString( MountainDT  );
    Pacifics  = TimeToString( PacificDT  );
    Locals    = TimeToString( Local_Time  ); 
    if(UTC <= BrokerDT+600 && UTC >= BrokerDT-600) {FLAG2_match = true;}
    
    //Since time zone data is already imported,
    //if vLines is on calculate GMT at Lo, Ny and Lc, 
    //if TzPivots is on calculate "Broker_GMT_Offset".     
    if(vLines_On && Period()<= vLines_Max_TF)
      {
      //Make 8am the "reference time" at GMT for London Open
      GMT_Ref_Time = 8; 
      
      //Find actual GMT At Lo (when London is at 8am local)             
      London_GMT_Offset = LondonDT/3600 - UTC/3600;      
      GMT_At_Lo = GMT_Ref_Time-London_GMT_Offset;       
               
      //Find actual GMT At Ny (when New York is at 8am local)
      NYC_GMT_Offset = NewYorkDT/3600 - UTC/3600;
      GMT_At_NYo = GMT_Ref_Time-NYC_GMT_Offset;
                        
      //Find actual GMT At Lc (nine hours after London Open)
      GMT_At_Lc = GMT_At_Lo + 9;                           
      }  
    
    if((vLines_On && Period()<= vLines_Max_TF) ||
      (TzPivots_On && Period()<= Daily_Main_Pivot_Max_TF))
      {
      //Define "Broker_GMT_Offset" for use with live feed
      if(!Apply_it_now) {Broker_GMT_Offset = BrokerDT/3600 - UTC/3600;}                       
      }
                 
    //Formating clock labels 
    FontName = "Arial";
    ClockFontSize = 7;
    TimezoneFontSize = 7;        
  
    //Populate Clock Labels           
    if(Broker == 1)  
      {
      if(!FLAG_match) {
      if(BrokerDT == Local_Time)  {Brokerp= "Broker  (L)";}                              
      else {Brokerp= "Broker";} 
      if(BrokerDT <= UTC + 600  && BrokerDT >= UTC - 600)
        {
        if(Brokerp== "Broker  (L)") 
          {
          Brokerp= StringSubstr(Brokerp,0,StringLen(Brokerp)-1) + ",GMT)";     
          } 
        else
          {
          Brokerp= StringSubstr(Brokerp,0) + "  (GMT)";
          } 
        }
      }       
	   ObjectSetText( "[CP Time] Brokert", Brokers, ClockFontSize, FontName, Clock_MktClosed );
      ObjectSetText( "[CP Time] Brokerl", Brokerp, TimezoneFontSize, FontName, Location_MktClosed );
      }
    if(Auckland == 1)                   
      {
      if(!FLAG_match) {
      if((AucklandDT <= BrokerDT+600 && AucklandDT >= BrokerDT-600)  &&
        (AucklandDT <= Local_Time+600 && AucklandDT >= Local_Time-600))
        {Aucklandp= "Auckland  (B,L)";}         
      else if ((AucklandDT <= BrokerDT+600) && (AucklandDT >= BrokerDT-600)) 
        {Aucklandp= "Auckland  (B)";}
      else if(AucklandDT == Local_Time) 
        {Aucklandp= "Auckland  (L)";}               
      else {Aucklandp= "Auckland";} }
   	if(TimeDayOfWeek(AucklandDT) != 0 && TimeDayOfWeek(AucklandDT) != 6 
   	  && TimeHour(AucklandDT) >= LocalOpenHour && TimeHour(AucklandDT) < LocalCloseHour)
        {
   	  ObjectSetText( "[CP Time] Aucklandt", Aucklands, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "[CP Time] Aucklandl", Aucklandp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
      else
   	  {
   	  ObjectSetText( "[CP Time] Aucklandt", Aucklands, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "[CP Time] Aucklandl", Aucklandp, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Sydney == 1)        
      {
      if(!FLAG_match) {
      if((SydneyDT <= BrokerDT+600 && SydneyDT >= BrokerDT-600)  &&
        (SydneyDT <= Local_Time+600 && SydneyDT >= Local_Time-600))
        {Sydneyp= "Sydney  (B,L)";}       
      else if ((SydneyDT <= BrokerDT+600) && (SydneyDT >= BrokerDT-600)) 
        {Sydneyp= "Sydney  (B)";}
      else if(SydneyDT == Local_Time) 
        {Sydneyp= "Sydney  (L)";}               
      else {Sydneyp= "Sydney";} }     	     	      	     
   	if(TimeDayOfWeek(SydneyDT) != 0 && TimeDayOfWeek(SydneyDT) != 6 
   	  && TimeHour(SydneyDT) >= SydneyLocalOpenHour && TimeHour(SydneyDT) < SydneyLocalCloseHour)
   	  {
   	  ObjectSetText( "[CP Time] Sydneyt", Sydneys, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "[CP Time] Sydneyl", Sydneyp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "[CP Time] Sydneyt", Sydneys, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "[CP Time] Sydneyl", Sydneyp, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Tokyo == 1)           
      {
      if(!FLAG_match) {
      if((TokyoDT <= BrokerDT+600 && TokyoDT >= BrokerDT-600)  &&
        (TokyoDT <= Local_Time+600 && TokyoDT >= Local_Time-600))
        {Tokyop= "Tokyo  (B,L)";}          
      else if ((TokyoDT <= BrokerDT+600) && (TokyoDT >= BrokerDT-600)) 
        {Tokyop= "Tokyo  (B)";}
      else if(TokyoDT == Local_Time) 
        {Tokyop= "Tokyo  (L)";}                 
      else {Tokyop= "Tokyo";} }        	      	         	         
   	if(TimeDayOfWeek(TokyoDT) != 0 && TimeDayOfWeek(TokyoDT) != 6 
   	  && TimeHour(TokyoDT) >= TokyoLocalOpenHour && TimeHour(TokyoDT) < TokyoLocalCloseHour)
   	  {
   	  ObjectSetText( "[CP Time] Tokyot", Tokyos, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "[CP Time] Tokyol", Tokyop, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "[CP Time] Tokyot", Tokyos, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "[CP Time] Tokyol", Tokyop, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(HongKong == 1)        
      {
      if(!FLAG_match) { 
      if((ChinaDT <= BrokerDT+600 && ChinaDT >= BrokerDT-600)  &&
        (ChinaDT <= Local_Time+600 && ChinaDT >= Local_Time-600))
        {Chinap= "HongKong  (B,L)";}          
      else if ((ChinaDT <= BrokerDT+600) && (ChinaDT >= BrokerDT-600)) 
        {Chinap= "HongKong  (B)";}
      else if(ChinaDT == Local_Time) 
        {Chinap= "HongKong  (L)";}                      
      else {Chinap= "HongKong";} }               	     	           
   	if(TimeDayOfWeek(ChinaDT) != 0 && TimeDayOfWeek(ChinaDT) != 6 
   	  && TimeHour(ChinaDT) >= LocalOpenHour && TimeHour(ChinaDT) < LocalCloseHour)
   	  {
    	  ObjectSetText( "[CP Time] Chinat", Chinas, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "[CP Time] Chinal", Chinap, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "[CP Time] Chinat", Chinas, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "[CP Time] Chinal", Chinap, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }      
    if(Jakarta == 1)   
      {
      if(!FLAG_match) { 
      if((JakartaDT <= BrokerDT+600 && JakartaDT >= BrokerDT-600)  &&
        (JakartaDT <= Local_Time+600 && JakartaDT >= Local_Time-600))
        {Jakartap= "Jakarta  (B,L)";}            
      else if ((JakartaDT <= BrokerDT+600) && (JakartaDT >= BrokerDT-600)) 
        {Jakartap= "Jakarta  (B)";}
      else if(JakartaDT == Local_Time) 
        {Jakartap= "Jakarta  (L)";}                   
      else {Jakartap= "Jakarta";} }            	     	         	     
   	if(TimeDayOfWeek(JakartaDT) != 0 && TimeDayOfWeek(JakartaDT) != 6
   	  && TimeHour(JakartaDT) >= LocalOpenHour && TimeHour(JakartaDT) < LocalCloseHour)
   	  {
   	  ObjectSetText( "[CP Time] Jakartat", Jakartas, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "[CP Time] Jakartal", Jakartap, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "[CP Time] Jakartat", Jakartas, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "[CP Time] Jakartal", Jakartap, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(India == 1)      
      {
      if(!FLAG_match) {   
      if((IndiaDT <= BrokerDT+600 && IndiaDT >= BrokerDT-600)  &&
        (IndiaDT <= Local_Time+600 && IndiaDT >= Local_Time-600))
        {Indiap= "India  (B,L)";}           
      else if ((IndiaDT <= BrokerDT+600) && (IndiaDT >= BrokerDT-600)) 
        {Indiap= "India  (B)";}
      else if(IndiaDT == Local_Time) 
        {Indiap= "India  (L)";}                     
      else {Indiap= "India";} }                  	       	     
   	if(TimeDayOfWeek(IndiaDT) != 0 && TimeDayOfWeek(IndiaDT) != 6 
   	  && TimeHour(IndiaDT) >= LocalOpenHour && TimeHour(IndiaDT) < LocalCloseHour)
   	  {
    	  ObjectSetText( "[CP Time] Indiat", Indias, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "[CP Time] Indial", Indiap, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
      else
        {
    	  ObjectSetText( "[CP Time] Indiat", Indias, ClockFontSize, FontName, Clock_MktClosed );     	
   	  ObjectSetText( "[CP Time] Indial", Indiap, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Dubai == 1)   
      {
      if(!FLAG_match) { 
      if((DubaiDT <= BrokerDT+600 && DubaiDT >= BrokerDT-600)  &&
        (DubaiDT <= Local_Time+600 && DubaiDT >= Local_Time-600))
        {Dubaip= "Dubai  (B,L)";}           
      else if ((DubaiDT <= BrokerDT+600) && (DubaiDT >= BrokerDT-600)) 
        {Dubaip= "Dubai  (B)";}
      else if(DubaiDT == Local_Time) 
        {Dubaip= "Dubai  (L)";}                     
      else {Dubaip= "Dubai";} } 	     	      	       	     
   	if(TimeDayOfWeek(DubaiDT) != 0 && TimeDayOfWeek(DubaiDT) != 6 
   	  && TimeHour(DubaiDT) >= LocalOpenHour && TimeHour(DubaiDT) < LocalCloseHour)
        {
   	  ObjectSetText( "[CP Time] Dubait", Dubais, ClockFontSize, FontName, Clock_MktOpen );     	
   	  ObjectSetText( "[CP Time] Dubail", Dubaip, TimezoneFontSize, FontName, Location_MktOpen ); 
   	  }
   	else
   	  {
   	  ObjectSetText( "[CP Time] Dubait", Dubais, ClockFontSize, FontName, Clock_MktClosed );  	   
   	  ObjectSetText( "[CP Time] Dubail", Dubaip, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Moscow == 1)              
      {
      if(!FLAG_match) {  
      if((MoscowDT <= BrokerDT+600 && MoscowDT >= BrokerDT-600)  &&
        (MoscowDT <= Local_Time+600 && MoscowDT >= Local_Time-600))
        {Moscowp= "Moscow  (B,L)";}           
      else if ((MoscowDT <= BrokerDT+600) && (MoscowDT >= BrokerDT-600)) 
        {Moscowp= "Moscow  (B)";}
      else if(MoscowDT == Local_Time) 
        {Moscowp= "Moscow  (L)";}                      
      else {Moscowp= "Moscow";} }     	           	      	        	       
   	if(TimeDayOfWeek(MoscowDT) != 0 && TimeDayOfWeek(MoscowDT) != 6 
   	  && TimeHour(MoscowDT) >= LocalOpenHour && TimeHour(MoscowDT) < LocalCloseHour)
   	  {
   	  ObjectSetText( "[CP Time] Moscowt", Moscows, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "[CP Time] Moscowl", Moscowp, TimezoneFontSize, FontName, Location_MktOpen ); 
        }
   	else
   	  {
    	  ObjectSetText( "[CP Time] Moscowt", Moscows, ClockFontSize, FontName, Clock_MktClosed );  	   
   	  ObjectSetText( "[CP Time] Moscowl", Moscowp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Israel == 1)                          
      {
      if(!FLAG_match) {  
      if((IsraelDT <= BrokerDT+600 && IsraelDT >= BrokerDT-600)  &&
        (IsraelDT <= Local_Time+600 && IsraelDT >= Local_Time-600))
        {Israelp= "Israel (B,L)";}          
      else if ((IsraelDT <= BrokerDT+600) && (IsraelDT >= BrokerDT-600)) 
        {Israelp= "Israel  (B)";}
      else if(IsraelDT == Local_Time) 
        {Israelp= "Israel  (L)";}                  
      else {Israelp= "Israel";} }              	     	         	    
   	if(TimeDayOfWeek(IsraelDT) != 0 && TimeDayOfWeek(IsraelDT) != 6
   	  && TimeHour(IsraelDT) >= LocalOpenHour && TimeHour(IsraelDT) < LocalCloseHour)
        {
   	  ObjectSetText( "[CP Time] Israelt", Israels, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "[CP Time] Israell", Israelp, TimezoneFontSize, FontName, Location_MktOpen );
        }
   	else
        {
  	     ObjectSetText( "[CP Time] Israelt", Israels, ClockFontSize, FontName, Clock_MktClosed );       	
   	  ObjectSetText( "[CP Time] Israell", Israelp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      } 
    if(Helsinki == 1)
      {
      if(!FLAG_match) {                         
      if((HelsinkiDT <= BrokerDT+600 && HelsinkiDT >= BrokerDT-600)  &&
        (HelsinkiDT <= Local_Time+600 && HelsinkiDT >= Local_Time-600))
        {Helsinkip= "Helsinki  (B,L)";}         
      else if((HelsinkiDT<=BrokerDT+600 && HelsinkiDT>=BrokerDT-600)) 
        {Helsinkip= "Helsinki  (B)";}
      else if(HelsinkiDT == Local_Time) 
        {Helsinkip= "Helsinki  (L)";}                  
      else {Helsinkip= "Helsinki";} }	                             	       	                         	                             	                 
   	if(TimeDayOfWeek(HelsinkiDT) != 0 && TimeDayOfWeek(HelsinkiDT) != 6 
   	  && TimeHour(HelsinkiDT) >= LocalOpenHour && TimeHour(HelsinkiDT) < LocalCloseHour)
        {
   	  ObjectSetText( "[CP Time] Helsinkit", Helsinkis, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "[CP Time] Helsinkil", Helsinkip, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
    	  ObjectSetText( "[CP Time] Helsinkit", Helsinkis, ClockFontSize, FontName, Clock_MktClosed );  	   
   	  ObjectSetText( "[CP Time] Helsinkil", Helsinkip, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Frankfurt == 1)                         
      {
      if(!FLAG_match) {    
      if((BerlinDT <= BrokerDT+600 && BerlinDT >= BrokerDT-600)  &&
        (BerlinDT <= Local_Time+600 && BerlinDT >= Local_Time-600))
        {Berlinp= "Frankfurt  (B,L)";}         
      else if ((BerlinDT <= BrokerDT+600) && (BerlinDT >= BrokerDT-600)) 
        {Berlinp= "Frankfurt  (B)";}
      else if(BerlinDT == Local_Time) 
        {Berlinp= "Frankfurt  (L)";}               
      else {Berlinp= "Frankfurt";} }                          
   	if(TimeDayOfWeek(BerlinDT) != 0 && TimeDayOfWeek(BerlinDT) != 6 
   	  && TimeHour(BerlinDT) >= LocalOpenHour && TimeHour(BerlinDT) < LocalCloseHour)
        {
   	  ObjectSetText( "[CP Time] Berlint", Berlins, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "[CP Time] Berlinl", Berlinp, TimezoneFontSize, FontName, Location_MktOpen );
        }
   	else
        {
   	  ObjectSetText( "[CP Time] Berlint", Berlins, ClockFontSize, FontName, Clock_MktClosed );     	
   	  ObjectSetText( "[CP Time] Berlinl", Berlinp, TimezoneFontSize, FontName, Location_MktClosed ); 
        }
      } 
    if(London == 1)            
      {
      if(!FLAG_match) {     
      if((LondonDT <= BrokerDT+600 && LondonDT >= BrokerDT-600)  &&
        (LondonDT <= Local_Time+600 && LondonDT >= Local_Time-600))
        {Londonp= "London  (B,L)";}         
      else if ((LondonDT <= BrokerDT+600) && (LondonDT >= BrokerDT-600)) 
        {Londonp= "London  (B)";}
      else if(LondonDT == Local_Time) 
        {Londonp= "London  (L)";}               
      else {Londonp= "London";} //}
      //if (LondonDT == UTC + 3600) //test code during summer
      if(LondonDT == UTC)
        {
        if((Londonp== "London  (B)") || (Londonp== "London  (L)") || (Londonp== "London  (B,L)"))
          {
          Londonp= StringSubstr(Londonp,0,StringLen(Londonp)-1) + ",G)";             
          }
        else
          {
          Londonp= StringSubstr(Londonp,0) + "  (G)";
          }   
        }
      //Show London as B+1 if Broker @ GMT but GMT not shown and if London= GMT+1
      if((GMT == 0) && (LondonDT != UTC) && (FLAG2_match))
        {
        if(Londonp== "London  (L)")
          {
          Londonp= StringSubstr(Londonp,0,StringLen(Londonp)-1) + ",B+1)";             
          }
        if(Londonp== "London")
          {
          Londonp= Londonp + "  (B+1)";             
          }  
        }}                                               
   	if(TimeDayOfWeek(LondonDT) != 0 && TimeDayOfWeek(LondonDT) != 6 
   	  && TimeHour(LondonDT) >= LocalOpenHour && TimeHour(LondonDT) < LocalCloseHour)
   	  {
        ObjectSetText( "[CP Time] Londont", Londons, ClockFontSize, FontName, Clock_MktOpen );   	   
        ObjectSetText( "[CP Time] Londonl", Londonp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "[CP Time] Londont", Londons, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "[CP Time] Londonl", Londonp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(GMT == 1)                
      {
      if(!FLAG_match) {        
      if((UTC <= BrokerDT+600 && UTC >= BrokerDT-600)  &&
        (UTC <= Local_Time+600 && UTC >= -600))
        {UTCp= "GMT  (B,L)";}           
      else if ((UTC <= BrokerDT+600) && (UTC >= BrokerDT-600)) 
        {UTCp= "GMT  (B)";}     
      else if (UTC== Local_Time) 
        {UTCp= "GMT  (L)";}                     	   
      else {UTCp= "GMT";} }     	              
   	if(TimeDayOfWeek(UTC) != 0 && TimeDayOfWeek(UTC) != 6 
   	  && TimeHour(UTC) >= LocalOpenHour && TimeHour(UTC) < LocalCloseHour)
   	  {
        ObjectSetText( "[CP Time] utct", UTCs, ClockFontSize, FontName, Clock_MktOpen );   	   
        ObjectSetText( "[CP Time] utcl", UTCp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {         
   	  ObjectSetText( "[CP Time] utct", UTCs, ClockFontSize, FontName, Clock_MktClosed );                    	         
   	  ObjectSetText( "[CP Time] utcl", UTCp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Brazil == 1)           
      {
      if(!FLAG_match) {  
      if((BrazilDT <= BrokerDT+600 && BrazilDT >= BrokerDT-600)  &&
        (BrazilDT <= Local_Time+600 && BrazilDT >= Local_Time-600))
        {Brazilp= "Brazil  (B,L)";}            
      else if ((BrazilDT <= BrokerDT+600) && (BrazilDT >= BrokerDT-600)) 
        {Brazilp= "Brazil  (B)";}       
      else if (BrazilDT == Local_Time) 
        {Brazilp= "Brazil  (L)";}       
      else {Brazilp= "Brazil";} }                    	     
   	if(TimeDayOfWeek(BrazilDT) != 0 && TimeDayOfWeek(BrazilDT) != 6 
   	  && TimeHour(BrazilDT) >= LocalOpenHour && TimeHour(BrazilDT) < LocalCloseHour)
   	  {
    	  ObjectSetText( "[CP Time] Brazilt", Brazils, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "[CP Time] Brazill", Brazilp, TimezoneFontSize, FontName, Location_MktOpen );
        }
   	else
        {
   	  ObjectSetText( "[CP Time] Brazilt", Brazils, ClockFontSize, FontName, Clock_MktClosed );      	
   	  ObjectSetText( "[CP Time] Brazill", Brazilp, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(NewYork == 1)     
      {
      if(!FLAG_match) {       
      if((NewYorkDT <= BrokerDT+600 && NewYorkDT >= BrokerDT-600)  &&
        (NewYorkDT <= Local_Time+600 && NewYorkDT >= Local_Time-600))
        {NYp= "NewYork  (B,L)";}           
      else if ((NewYorkDT <= BrokerDT+600) && (NewYorkDT >= BrokerDT-600)) 
        {NYp= "NewYork  (B)";}       
      else if (NewYorkDT == Local_Time) 
        {NYp= "NewYork  (L)";}                         	   
      else {NYp= "NewYork";} }                         
   	if(TimeDayOfWeek(NewYorkDT) != 0 && TimeDayOfWeek(NewYorkDT) != 6 
   	  && TimeHour(NewYorkDT) >= LocalOpenHour && TimeHour(NewYorkDT) < LocalCloseHour)
   	  {
        ObjectSetText( "[CP Time] NewYorkt", NewYorks, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "[CP Time] NewYorkl", NYp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
        ObjectSetText( "[CP Time] NewYorkt", NewYorks, ClockFontSize, FontName, Clock_MktClosed );  	   
        ObjectSetText( "[CP Time] NewYorkl", NYp, TimezoneFontSize, FontName, Location_MktClosed ); 
        }
      }
    if(Central == 1)
      {             
      if(!FLAG_match) {
      if((CentralDT <= BrokerDT+600 && CentralDT >= BrokerDT-600)  &&
        (CentralDT <= Local_Time+600 && CentralDT >= Local_Time-600))
        {Centralp= "Central  (B,L)";}	          
      else if ((CentralDT <= BrokerDT+600) && (CentralDT >= BrokerDT-600)) 
        {Centralp= "Central  (B)";}     
      else if (CentralDT == Local_Time) 
        {Centralp= "Central  (L)";}      
      else {Centralp= "Central";} }                            	      
   	if(TimeDayOfWeek(CentralDT) != 0 && TimeDayOfWeek(CentralDT) != 6 
   	 && TimeHour(CentralDT) >= LocalOpenHour && TimeHour(CentralDT) < LocalCloseHour)
   	  {
  	     ObjectSetText( "[CP Time] Centralt", Centrals, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "[CP Time] Centrall", Centralp, TimezoneFontSize, FontName, Location_MktOpen ); 
   	  }
   	else
   	  {
  	     ObjectSetText( "[CP Time] Centralt", Centrals, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "[CP Time] Centrall", Centralp, TimezoneFontSize, FontName, Location_MktClosed ); 
   	  }
      }
    if(Mexico == 1)             
      {
      if(!FLAG_match) {
      if((MexicoDT <= BrokerDT+600 && MexicoDT >= BrokerDT-600)  &&
        (MexicoDT <= Local_Time+600 && MexicoDT >= Local_Time-600))
        {Mexicop= "Mexico  (B,L)";}        
      else if ((MexicoDT <= BrokerDT+600) && (MexicoDT >= BrokerDT-600)) 
        {Mexicop= "Mexico  (B)";}      
      else if (MexicoDT == Local_Time) 
        {Mexicop= "Mexico  (L)";}       
      else {Mexicop= "Mexico";} }                    	     
   	if(TimeDayOfWeek(MexicoDT) != 0 && TimeDayOfWeek(MexicoDT) != 6 
   	  && TimeHour(MexicoDT) >= LocalOpenHour && TimeHour(MexicoDT) < LocalCloseHour)
   	  {
   	  ObjectSetText( "[CP Time] Mexicot", Mexicos, ClockFontSize, FontName, Clock_MktOpen );   	   
   	  ObjectSetText( "[CP Time] Mexicol", Mexicop, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "[CP Time] Mexicot", Mexicos, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "[CP Time] Mexicol", Mexicop, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Mountain == 1)         
      {
      if(!FLAG_match) { 
      if((MountainDT <= BrokerDT+600 && MountainDT >= BrokerDT-600)  &&
        (MountainDT <= Local_Time+600 && MountainDT >= Local_Time-600))
        {Mountainp= "Mountain  (B,L)";}           
      else if ((MountainDT <= BrokerDT+600) && (MountainDT >= BrokerDT-600)) 
        {Mountainp= "Mountain  (B)";}      
      else if (MountainDT == Local_Time) 
        {Mountainp= "Mountain  (L)";}        
      else {Mountainp= "Mountain";} }       	                 	      
   	if(TimeDayOfWeek(MountainDT) != 0 && TimeDayOfWeek(MountainDT) != 6 
   	  && TimeHour(MountainDT) >= LocalOpenHour && TimeHour(MountainDT) < LocalCloseHour)
        {
  	     ObjectSetText( "[CP Time] Mountaint", Mountains, ClockFontSize, FontName, Clock_MktOpen );      	
   	  ObjectSetText( "[CP Time] Mountainl", Mountainp, TimezoneFontSize, FontName, Location_MktOpen );
   	  }
   	else
   	  {
   	  ObjectSetText( "[CP Time] Mountaint", Mountains, ClockFontSize, FontName, Clock_MktClosed );   	   
   	  ObjectSetText( "[CP Time] Mountainl", Mountainp, TimezoneFontSize, FontName, Location_MktClosed );
   	  }
      }
    if(Pacific == 1)                
      {
      if(!FLAG_match) { 
      if((PacificDT <= BrokerDT+600 && PacificDT >= BrokerDT-600)  &&
        (PacificDT <= Local_Time+600 && PacificDT >= Local_Time-600))
        {Pacificp= "Pacific  (B,L)";}           
      else if ((PacificDT <= BrokerDT+600) && (PacificDT >= BrokerDT-600)) 
        {Pacificp= "Pacific  (B)";}      
      else if(PacificDT == Local_Time) 
        {Pacificp= "Pacific  (L)";}     
      else {Pacificp= "Pacific";} }                       	     
   	if(TimeDayOfWeek(PacificDT) != 0 && TimeDayOfWeek(PacificDT) != 6 
   	  && TimeHour(PacificDT) >= LocalOpenHour && TimeHour(PacificDT) < LocalCloseHour)
   	  {
   	  ObjectSetText( "[CP Time] Pacifict", Pacifics, ClockFontSize, FontName, Clock_MktOpen );  	   
   	  ObjectSetText( "[CP Time] Pacificl", Pacificp, TimezoneFontSize, FontName, Location_MktOpen ); 
   	  }
   	else
        {
    	  ObjectSetText( "[CP Time] Pacifict", Pacifics, ClockFontSize, FontName, Clock_MktClosed );     	
   	  ObjectSetText( "[CP Time] Pacificl", Pacificp, TimezoneFontSize, FontName, Location_MktClosed );
        }
      }
    if(Local == 1)        
      {
      if(!FLAG_match) {
      if(Local_Time == BrokerDT)  {Localp= "Local  (B)";}                          
      else {Localp= "Local";} 
      if(Local_Time <= UTC + 600  && Local_Time >= UTC - 600)
        {
        if(Localp== "Local  (B)") 
          {
          Localp= StringSubstr(Localp,0,StringLen(Localp)-1) + ",GMT)";     
          } 
        else
          {
          Localp= StringSubstr(Localp,0) + "  (GMT)";
          } 
        }}         
      ObjectSetText( "[CP Time] Localt", Locals, ClockFontSize, FontName,Clock_MktClosed );      
      ObjectSetText( "[CP Time] Locall", Localp, TimezoneFontSize, FontName, Location_MktClosed ); 
      }   
    if(!FLAG_match) 
      {  
      if(StaticBrokerDT < BrokerDT) {FLAG_match = true;} 
      }
    }//End Clock           
     
  //Bid Line----------------------------------------------------------------------------------- 
  if(Bid_Line_On)
    {
    //Bid line
    T1=Time[0]+(Period()*60*Bid_Ray_TFs_Past_Last_Candle); 
    T2=iTime(NULL, NULL, (Period()*60));
    ObjectCreate(item18, OBJ_TREND, 0, T1, Bid, T2, Bid);     
    ObjectSet(item18, OBJPROP_STYLE, Bid_Ray_LineStyle_01234 );
    ObjectSet(item18, OBJPROP_COLOR, Bid_Ray_Color);
    ObjectSet(item18, OBJPROP_WIDTH, Bid_Ray_Thickness);
               
    //Bid dot                   
    T1=Time[0]+(Period()*60*(Bid_Ray_TFs_Past_Last_Candle));  
    ObjectCreate(item19, OBJ_TREND, 0, T1, Bid, T1, Bid);     
    ObjectSet(item19, OBJPROP_STYLE, 0);
    ObjectSet(item19, OBJPROP_COLOR, Bid_Dot_Color);
    ObjectSet(item19, OBJPROP_WIDTH, Bid_Dot_Size);                      
    }//End Bid Line
   
  //Levels-------------------------------------------------------------------------------------
  if(Levels_On && Period()<= Levels_Max_TF)
    {      
    //initializing line colors
    Line1_Color = Level_Whole_Line_Color; 
    Line2_Color = Level_Half_Line_Color;
    Line3_Color = Level_Quarter_Line_Color;
       
    //define needed variables
    ssp1= Bid / Poin;  //bid w/o the decimal
    v1=ssp1%100; //last two significant digits   
    BL=v1; //last two significant digits of starting/reference bid line  
       
    //calculate line levels bottom to top
    for(j= -(100-(100-BL))-(100*NumberOfRanges); j<= (100-BL)+(100*NumberOfRanges); j++)       
      {
      ssp=ssp1+j; 
      v1=ssp%100;        
      if(v1==u1)
        {           
        linestyle = Line1_Style_01234;
        linecolor = Line1_Color;
        linewidth = Line1_SolidThickness;
        linelevel = ssp*Poin;
        DrawLines("_LVL", Level_Whole_Display_Number, linelevel, 
        linecolor, linestyle, linewidth, Level_Whole_Shift_Line_Right);
        }
      if((Level_Half_Lines_On) && (v1==u2))
        {              
        linestyle = Line2_Style_01234;
        linecolor = Line2_Color;
        linewidth = Line2_SolidThickness;
        linelevel = ssp*Poin;
        DrawLines("_LVL", Level_Half_Display_Number, linelevel, 
        linecolor, linestyle, linewidth, Level_Whole_Shift_Line_Right);
        } 
      if((Level_Quarter_Lines_On) && ((v1==u3) || (v1==u4)))
        {           
        linestyle = Line3_Style_01234;
        linecolor = Line3_Color;
        linewidth = Line3_SolidThickness;
        linelevel = ssp*Poin;                        
        DrawLines("_LVL", Level_Quarter_Display_Number, linelevel, 
        linecolor, linestyle, linewidth, Level_Whole_Shift_Line_Right);
        }
      }
    }//End Levels         

  //Weekly Range High/Low lines---------------------------------------------------------------
  if(Week_Range_Lines_On && Period()<= Week_Range_Lines_Max_TF && 
    Period()>= Week_Range_Lines_Min_TF)
    {
    ARg = 0; for(i=1; i<=Weeks_For_Range_Week_Average; i++)
      {
      ARg = ARg + iHigh(NULL,PERIOD_W1,i)- iLow(NULL,PERIOD_W1,i);
      }
    wRangeAvg = NormalizeDouble(ARg/Weeks_For_Range_Week_Average,4); 
    
    Weekshift  = 0;  //iBarShift(NULL,PERIOD_W1,Time[0])+0;//today bar= +0, yesterday bar= +1	 
    HiWeek     = iHigh(NULL,PERIOD_W1,Weekshift);
	 LoWeek	   = iLow (NULL,PERIOD_W1,Weekshift);
	 wHbarshift = iHighest(NULL,0,MODE_HIGH,10080/Period(),Weekshift); 
	 wLbarshift = iLowest(NULL,0,MODE_LOW,10080/Period(),Weekshift);     	   	
           
    wRangeHigh = LoWeek + wRangeAvg;  
    wRangeLow  = HiWeek - wRangeAvg; 
        
    if(HiWeek - LoWeek > wRangeAvg)
      {
      if(rwc == 1) //Pass over first time, but do thereafter
        { 
        wRangeHigh = wRH;
        wRangeLow = wRL;        
        } 
      if(rwc == 0) //Do this the first and only time
        {            
        if(Time[wLbarshift] < Time[wHbarshift]) 
          {wRangeLow  = LoWeek; wRangeHigh = LoWeek + wRangeAvg;}
        else {wRangeHigh = HiWeek; wRangeLow = HiWeek - wRangeAvg;}              
        wRH = wRangeHigh;
        wRL = wRangeLow;
        rwc = 1;               
        }           
      }      
      
    //Range Lines data to subroutine                                                             
    DrawLines("RWH", Range_Week_Display_Number, wRangeHigh, RWH_Color, 
    W_LineStyle_01234, W_SolidLineThickness, Range_Week_Shift_Line_Right);   
    DrawLines("RWL", Range_Week_Display_Number, wRangeLow,  RWL_Color, 
    W_LineStyle_01234, W_SolidLineThickness, Range_Week_Shift_Line_Right);
    }//End Range Week
     
  //Daily Range High/Low lines-----------------------------------------------------------------
  if(Day_Range_Lines_On && Period()<= Day_Range_Lines_Max_TF)
    {       
    Ranges(Days_For_Range_Day_Average);           
    RangeAvg = NormalizeDouble(ARg/Days_For_Range_Day_Average,4);              
    RangeHigh = LoToday + RangeAvg;  
    RangeLow  = HiToday - RangeAvg; 
    
    if(HiToday - LoToday > RangeAvg)
      {
      if(rdc == 1) //Pass over first time, but do thereafter
        { 
        RangeHigh = RH;
        RangeLow = RL;        
        } 
      if(rdc == 0) //Do this the first and only time
        {            
        if(Time[Lbarshift] < Time[Hbarshift]) 
          {RangeLow  = LoToday; RangeHigh = LoToday + RangeAvg;}
        else {RangeHigh = HiToday; RangeLow = HiToday - RangeAvg;}              
        RH = RangeHigh;
        RL = RangeLow;
        rdc = 1;               
        }           
      }
                           
    //Range Lines data to subroutine                                                             
    DrawLines("RDH", Range_Day_Display_Number, RangeHigh, RDH_Color, 
    D_LineStyle_01234, D_SolidLineThickness, Range_Day_Shift_Line_Right);   
    DrawLines("RDL", Range_Day_Display_Number, RangeLow,  RDL_Color, 
    D_LineStyle_01234, D_SolidLineThickness, Range_Day_Shift_Line_Right);
    }//End Range Day
                         
  //Daily Pivot Lines-------------------------------------------------------------------------- 
  if((Daily_Main_Pivot_On && Period()<= Daily_Main_Pivot_Max_TF) || 
    (Daily_SR_Pivot_Lines_On && Period()<= Daily_SR_Pivots_Max_TF))             
    {
    //CaveMan's sophisticated Tz shift code finds out which hour bars start "today" and 
    //"yesterday" in the time zone selected for the pivots calculations.  The Tz code needs 
    //"Broker_GMT_Offset" and "TzPivots_GMT_Offset".  The "Broker_GMT_Offset"
    //is needed because unless the server is permanently at GMT, it will vary in offset to GMT
    //depending if it is the DST or non-DST time of the year.  The difference between the two
    //items is used by the Tz shift code to shift away from the broker server hour bar that
    //starts the current day, thus defining the start of the current day for use in the time 
    //zone shifted pivot calculations. 
    
    //If Broker/Server Pivots to be used make both items same (yields broker pivots)
    if(!TzPivots_On) {TzPivots_GMT_Offset = Broker_GMT_Offset;}
    
    //Send Items to the TzPivots coding subroutine                            
    TzShift(Broker_GMT_Offset, TzPivots_GMT_Offset,
    idxfirstbartoday, idxfirstbaryesterday,idxlastbaryesterday);
       
    //Walk forward through yestday's start and collect high/lows within the same day
    yesterday_high= -99999;  // not high enough to remain alltime high
    yesterday_low=  +99999;  // not low enough to remain alltime low 
    for(int idxbar= idxfirstbaryesterday; idxbar>=idxlastbaryesterday; idxbar--)
      {
      // grab first value for open
      if (yesterday_open==0) yesterday_open= iOpen(NULL, PERIOD_H1, idxbar);      
      yesterday_high= MathMax(iHigh(NULL, PERIOD_H1, idxbar), yesterday_high);
      yesterday_low= MathMin(iLow(NULL, PERIOD_H1, idxbar), yesterday_low);      
      yesterday_close= iClose(NULL, PERIOD_H1, idxbar);
      }
         
    d = (today_high - today_low);
    q = (yesterday_high - yesterday_low);
    p = (yesterday_high + yesterday_low + yesterday_close) / 3;                          
     
    if (Daily_Pivots_Use_Fib_Formula) {prefix = "F";}
    else {prefix = "D";} 
               
    if(Daily_SR_Pivot_Lines_On)
      {       
      if(Daily_Pivots_Use_Fib_Formula)
        {
        r5 = p+ (q * 2.618);
        r4 = p+ (q * 1.618);
  	     r3 = p+q;  
        r2 = p+ (q * 0.618); 
        r1 = p+ (q * 0.382); 
        s1 = p- (q * 0.382);     
        s2 = p- (q * 0.618);    
  	     s3 = p-q; 
        s4 = p- (q * 1.618);
        s5 = p- (q * 2.618);  
        }
      else   
        {
        r5 = (4*p)+(yesterday_high-(4*yesterday_low));
        r4 = (3*p)+(yesterday_high-(3*yesterday_low));
        r3 = (2*p)+(yesterday_high-(2*yesterday_low));
        r2 = p+(yesterday_high - yesterday_low);  //r2 = p-s1+r1;
        r1 = (2*p)-yesterday_low;  
        s1 = (2*p)-yesterday_high;
        s2 = p-(yesterday_high - yesterday_low);  //s2 = p-r1+s1;
        s3 = (2*p)-((2* yesterday_high)-yesterday_low);
        s4 = (3*p)-((3* yesterday_high)-yesterday_low);
        s5 = (4*p)-((4* yesterday_high)-yesterday_low);
        }                
      }
                      
    //Daily Central Pivot Line 
    if(Daily_Main_Pivot_On && Period()<= Daily_Main_Pivot_Max_TF)
      {         
      //Pivot Line data to subroutine                                                               
      DrawLines("DPV1", Pivot_Main_Display_Number, p, DPV1_Background_Color, 
      DPV1_LineStyle_01234, DPV1_SolidLineThickness, Pivot_Main_Shift_Line_Right);
      DrawLines("DPV2", Pivot_Main_Display_Number, p, DPV2_Top_Color, 
      DPV2_LineStyle_01234, DPV2_SolidLineThickness, Pivot_Main_Shift_Line_Right);         
      }
      
    //Daily Support & Resistance Lines   
    if(Daily_SR_Pivot_Lines_On && Period()<= Daily_SR_Pivots_Max_TF)
      {          
      DrawLines(prefix+"R5", Pivot_SR_Display_Number, r5, Pivot_Resistance_Color, 
      R_LineStyle_01234, R_SolidLineThickness, Pivot_SR_Shift_Line_Right);  
      DrawLines(prefix+"R4", Pivot_SR_Display_Number, r4, Pivot_Resistance_Color, 
      R_LineStyle_01234, R_SolidLineThickness, Pivot_SR_Shift_Line_Right);   
      DrawLines(prefix+"R3", Pivot_SR_Display_Number, r3, Pivot_Resistance_Color, 
      R_LineStyle_01234, R_SolidLineThickness, Pivot_SR_Shift_Line_Right);   
      DrawLines(prefix+"R2", Pivot_SR_Display_Number, r2, Pivot_Resistance_Color, 
      R_LineStyle_01234, R_SolidLineThickness, Pivot_SR_Shift_Line_Right); 
      DrawLines(prefix+"R1", Pivot_SR_Display_Number, r1, Pivot_Resistance_Color, 
      R_LineStyle_01234, R_SolidLineThickness, Pivot_SR_Shift_Line_Right);   
      DrawLines(prefix+"S1", Pivot_SR_Display_Number, s1, Pivot_Support_Color, 
      S_LineStyle_01234, S_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines(prefix+"S2", Pivot_SR_Display_Number, s2, Pivot_Support_Color, 
      S_LineStyle_01234, S_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines(prefix+"S3", Pivot_SR_Display_Number, s3, Pivot_Support_Color, 
      S_LineStyle_01234, S_SolidLineThickness, Pivot_SR_Shift_Line_Right);  
      DrawLines(prefix+"S4", Pivot_SR_Display_Number, s4, Pivot_Support_Color, 
      S_LineStyle_01234, S_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines(prefix+"S5", Pivot_SR_Display_Number, s5, Pivot_Support_Color, 
      S_LineStyle_01234, S_SolidLineThickness, Pivot_SR_Shift_Line_Right);              
      }
    //Mid-Pivots
    if(MidPivots_On && Period()<= Daily_SR_Pivots_Max_TF) 
      {   
      //if(mP_LineStyle_01234>0){mP_SolidLineThickness=0;}
      DrawLines("MR5", Pivot_SR_Display_Number, (r4+r5)/2, MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines("MR4", Pivot_SR_Display_Number, (r3+r4)/2, MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right); 
      DrawLines("MR3", Pivot_SR_Display_Number, (r2+r3)/2, MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines("MR2", Pivot_SR_Display_Number, (r1+r2)/2, MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines("MR1", Pivot_SR_Display_Number, (p+r1)/2,  MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines("MS1", Pivot_SR_Display_Number, (p+s1)/2,  MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines("MS2", Pivot_SR_Display_Number, (s1+s2)/2, MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines("MS3", Pivot_SR_Display_Number, (s2+s3)/2, MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right);          
      DrawLines("MS4", Pivot_SR_Display_Number, (s3+s4)/2, MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      DrawLines("MS5", Pivot_SR_Display_Number, (s4+s5)/2, MidPivots_Color, 
      mP_LineStyle_01234, mP_SolidLineThickness, Pivot_SR_Shift_Line_Right);
      }                             
    }//End Daily Pivots
   
  //Separators---------------------------------------------------------------------------------   
  if(Separators_On && Period() <= Separators_Max_TF) 
    {
    //Separators for Today and Tomorrow           
    CurrentSeparators("Today", Separators_Color, Separators_Style_01234, 
    Separators_Solid_Thickness); 
             
    CurrentSeparators("Tomorrow", Separators_Color, Separators_Style_01234, 
    Separators_Solid_Thickness);  
              
    //Separators for prior sessions 
    if(Prior_Days_To_Show_Separators > 0)
      {     
      //Calculate bars per session        
           if (Period()==1) {bps = 1440;}   
      else if (Period()==5) {bps = 288;}  
      else if (Period()==15){bps = 96;}    
      else if (Period()==30){bps = 48;}
      else if (Period()==60){bps = 24;}
      else if (Period()==240){bps = 6;}         
      //Define bar starting prior sessions display      
      shift = iBarShift(NULL,NULL,iTime(NULL,PERIOD_D1,0));                       
      //Execute loop for bars per session X number of prior sessions to show   
      for(i= shift; i<=(shift+(bps*(Prior_Days_To_Show_Separators))); i++)   
        {    
        if(TimeHour(Time[i]) == 0 && TimeMinute(Time[i]) == 0)         
          {                     
          PriorSeparators(i, Separators_Color, 
          Separators_Style_01234, Separators_Solid_Thickness);                             
          }         
        }               
      }
    }//End Separators   

  //vLines-------------------------------------------------------------------------------------
  if(vLines_On && Period()<= vLines_Max_TF)
    {                 
    //Calculate position for vline labels------------------------------------------------------
    if(Show_vLabels)
      {  
      if (vLabels_Dist_To_Border < 1) {vLabels_Dist_To_Border = 1;}           
      top = WindowPriceMax();
      bottom = WindowPriceMin();
      scale = top - bottom; 
      if(vLabels_Chart_Top) 
        {           
        YadjustTop = scale/(13000/vLabels_FontSize);     
        YadjustTop = YadjustTop + (vLabels_Dist_To_Border * YadjustTop);                          	      	
        level = top - YadjustTop;
        }                 
      else if(!vLabels_Chart_Top) 
        {
        vLabels_Dist_Above_Border = vLabels_Dist_To_Border + 3;
        YadjustBot = scale/(500/vLabels_FontSize); 
        YadjustBot = YadjustBot + ((vLabels_Dist_Above_Border * YadjustBot)/20);   
        level = bottom + YadjustBot;
        }  
      }       
            
    //Do vlines for current session------------------------------------------------------------        
    if(!(DayOfWeek() == 0 && Broker_GMT_Offset == 0)) 
      {        
      OpenToday(Label_London_Open, GMT_At_Lo + Broker_GMT_Offset, 
      vLines_Color, vLines_Style_01234, vLines_Solid_Thickness, level);

      OpenToday(Label_NewYork_Open, GMT_At_NYo + Broker_GMT_Offset,  
      vLines_Color,vLines_Style_01234, vLines_Solid_Thickness, level);

      OpenToday(Label_London_Close, GMT_At_Lc + Broker_GMT_Offset, 
      vLines_Color, vLines_Style_01234, vLines_Solid_Thickness, level);  
      }                 
                    
    //Do vlines for previous sessions----------------------------------------------------------   
    if(vLines_Prior_Sessions_To_Show >0) 
      {   
      //Calculate bars per session        
           if (Period()==1) {bps = 1440;}   
      else if (Period()==5) {bps = 288;}  
      else if (Period()==15){bps = 96;}    
      else if (Period()==30){bps = 48;}
      else if (Period()==60){bps = 24;}         
      //Define bar starting prior sessions display      
      shift = iBarShift(NULL,NULL,iTime(NULL,PERIOD_D1,0));               
      //Execute loop for bars per session X number of prior sessions to show    
      for(i= shift; i<=(shift+(bps*(vLines_Prior_Sessions_To_Show))); i++)       
        {  
        h=TimeHour(Time[i]); 
        m=TimeMinute(Time[i]);
               
        //London Open vLine
        if(GMT_At_Lo+Broker_GMT_Offset<0) {GMT_At_Lo=GMT_At_Lo+24;}                
        MF = MathFloor(GMT_At_Lo); 
        if((Period() > 30) && (h == MF + Broker_GMT_Offset) && (m == 0))                
          {
          OpenPrior(i, Label_London_Open, vLines_Color, vLines_Style_01234, 
          vLines_Solid_Thickness, level);
          }               
        else if((GMT_At_Lo != MF) && (h == MF + Broker_GMT_Offset) && (m == 30)) 
          {                    
          OpenPrior(i, Label_London_Open, vLines_Color, vLines_Style_01234, 
          vLines_Solid_Thickness, level);
          }                   
        else if((GMT_At_Lo == MF) && (h == MF + Broker_GMT_Offset) && (m == 0))
          {
          OpenPrior(i, Label_London_Open, vLines_Color, vLines_Style_01234, 
          vLines_Solid_Thickness, level);
          }   
         
        //NewYork Open vLine             
        if(GMT_At_NYo+Broker_GMT_Offset<0) {GMT_At_NYo=GMT_At_NYo+24;}                
        MF = MathFloor(GMT_At_NYo); 
        if((Period() > 30) && (h == MF + Broker_GMT_Offset) && (m == 0))                
          {
          OpenPrior(i, Label_NewYork_Open, vLines_Color, vLines_Style_01234, 
          vLines_Solid_Thickness, level);
          }                  
        else if((GMT_At_NYo != MF) && (h == MF + Broker_GMT_Offset) && (m == 30)) 
          {                    
          OpenPrior(i, Label_NewYork_Open, vLines_Color, vLines_Style_01234, 
          vLines_Solid_Thickness, level);
          }                   
        else if((GMT_At_NYo == MF) && (h == MF + Broker_GMT_Offset) && (m == 0))               
          {
          OpenPrior(i, Label_NewYork_Open, vLines_Color, vLines_Style_01234, 
          vLines_Solid_Thickness, level);
          }              
               
        //London Close vLine           
        if(GMT_At_Lc+Broker_GMT_Offset<0) {GMT_At_Lc=GMT_At_Lc+24;}                                                      
        MF = MathFloor(GMT_At_Lc); 
        if((Period() > 30) && (h == MF + Broker_GMT_Offset) && (m == 0))                
          {
          OpenPrior(i, Label_London_Close, vLines_Color, vLines_Style_01234, 
          vLines_Solid_Thickness, level);
          }                   
        else if((GMT_At_Lc != MF) && (h == MF + Broker_GMT_Offset) && (m == 30)) 
          {                    
          OpenPrior(i, Label_London_Close, vLines_Color, vLines_Style_01234, 
          vLines_Solid_Thickness, level);
          }                   
        else if((GMT_At_Lc == MF) && (h == MF + Broker_GMT_Offset) && (m == 0))               
          {
          OpenPrior(i, Label_London_Close, vLines_Color, vLines_Style_01234, 
          vLines_Solid_Thickness, level);
          }                                                                          
        }//End prior sessions shift loop                            
      }//End prior sessions routine                          
    }//End vLines routine
  
   //End of program computations---------------------------------------------------------------        
  return(0);
  }

//+-------------------------------------------------------------------------------------------+
//| Indicator Subroutine To Compute Average Ranges                                            |                                                 
//+-------------------------------------------------------------------------------------------+ 
void Ranges (int period)
  {  
  int x, xx;
  ARg = 0;
  
  //Add ranges over period.  Count number of Sundays and exclude Sunday ranges.             
  for(i=1; i<=period; i++)
    {
    if (TimeDayOfWeek(iTime(NULL,PERIOD_D1,i))!=0) {
    ARg = ARg + iHigh(NULL,PERIOD_D1,i)- iLow(NULL,PERIOD_D1,i);}
    else {x=x+1;}
    }

  //For number of Sundays, add additional days of range
  for(ii=i; ii<=i-1+x; ii++) 
    { 
    if (TimeDayOfWeek(iTime(NULL,PERIOD_D1,ii))!=0) 
      {       
      ARg = ARg + iHigh(NULL,PERIOD_D1,ii)- iLow(NULL,PERIOD_D1,ii);
      }
    else {xx=xx+1;} 
    }
   
  //If a Sunday reduced added days above, add additional day of range
  for(iii=ii; iii<=ii-1+xx; iii++)
    {
    ARg = ARg + iHigh(NULL,PERIOD_D1,iii)- iLow(NULL,PERIOD_D1,iii);
    }
                        
  return (ARg);   
  }
  
//+-------------------------------------------------------------------------------------------+
//| Subroutine To name and draw Daily Pivot and Range HL lines and line labels                |                                                                                  |
//+-------------------------------------------------------------------------------------------+
void DrawLines(string text, int Line_Selection_Number, double level, color Color, 
  int linestyle, int thickness, int Shift_Line_Right)
  {
  //Lines====================================================================================== 
  //Lines & Labels - Set values to Shift_Line_Right if Line_Selection_Number is "4"
  if(Line_Selection_Number == 4) 
    {  
    if(Shift_Line_Right == 1)
      {
      Zoom_Out_2  = 78;
      Zoom_Out_1  = 40; 
      Zoom_Std    = 19; 
      Zoom_In_1   = 12; 
      Zoom_In_2   = 7; 
      Zoom_In_3   = 4;    
      }
    else if (Shift_Line_Right == 2)
      {
      Zoom_Out_2 = 112;
      Zoom_Out_1  = 58; 
      Zoom_Std    = 28; 
      Zoom_In_1   = 16; 
      Zoom_In_2   = 9; 
      Zoom_In_3   = 5; 
      }
    else if (Shift_Line_Right == 3)
      {
      Zoom_Out_2 = 144;
      Zoom_Out_1 = 74; 
      Zoom_Std   = 37; 
      Zoom_In_1  = 20; 
      Zoom_In_2  = 11; 
      Zoom_In_3  = 6;    
      }   
    else if (Shift_Line_Right == 4)
      {
      Zoom_Out_2 = 178; 
      Zoom_Out_1 = 91; 
      Zoom_Std   = 46;  
      Zoom_In_1  = 24;  
      Zoom_In_2  = 13; 
      Zoom_In_3  = 7;  
      }   
    else if (Shift_Line_Right == 5)
      {
      Zoom_Out_2 = 211;
      Zoom_Out_1 = 107; 
      Zoom_Std   = 54; 
      Zoom_In_1  = 28;
      Zoom_In_2  = 15; 
      Zoom_In_3  = 8;     
      }
    if((Daily_SR_Pivot_Lines_On && Pivot_SR_Display_Number == 4) && 
      ((StringSubstr(text, 0, 2) == "RD") || (StringSubstr(text, 0, 2) == "RW")) &&
      ((Pivot_SR_Shift_Line_Right == Range_Day_Shift_Line_Right) ||
      (Pivot_SR_Shift_Line_Right == Range_Week_Shift_Line_Right)))   
      { 
      Zoom_Out_2 = Zoom_Out_2-30;
      Zoom_Out_1 = Zoom_Out_1-15; 
      Zoom_Std   = Zoom_Std-8; 
      Zoom_In_1  = Zoom_In_1-4;
      Zoom_In_2  = Zoom_In_2-2; 
      Zoom_In_3  = Zoom_In_3-1;   
      } 
    }             
          
  //Name lines 
  text = StringTrimLeft(text);       
  line = "[CP Art]  " + text + " Line " + DoubleToStr(level,Digits);  
    
  //Define variables  
  a = linestyle; b = thickness; c =1; if (a==0)c=b;  
   
  //Where to Start lines
  //start chart left
  if((Line_Selection_Number == 0) || (Line_Selection_Number == 1)) 
    {startline = WindowFirstVisibleBar();}
  //start at day separator 
  else if(Line_Selection_Number == 2 || Line_Selection_Number == 3) 
    {startline = iTime(NULL, PERIOD_D1, 0);}
  //start at current candle, or further to the right of the current candle                     
  else if(Line_Selection_Number == 4)
    {
    //start right of last candle
    if (Shift_Line_Right == 0) {startline = Time[0];} 
    //start right of last candle + shift right value
    else
      { 
           if (WindowBarsPerChart( ) >= Zoom_Out_2_Bars) {T4 = Period()*60*Zoom_Out_2;} 
      else if (WindowBarsPerChart( ) >= Zoom_Out_1_Bars) {T4 = Period()*60*Zoom_Out_1;} 
      else if (WindowBarsPerChart( ) >= Zoom_Std_Bars)   {T4 = Period()*60*Zoom_Std;} 
      else if (WindowBarsPerChart( ) >= Zoom_In_1_Bars)  {T4 = Period()*60*Zoom_In_1;} 
      else if (WindowBarsPerChart( ) >= Zoom_In_2_Bars)  {T4 = Period()*60*Zoom_In_2;} 
      else {T4 = Period()*60*Zoom_In_3;}                  
      startline = Time[0]+ T4;  
      }
    }
                                                                              
  //Where to Stop Lines   
  //stop at chart right border
  if(Line_Selection_Number == 0 || Line_Selection_Number == 3) 
    {stopline = iTime(NULL,PERIOD_D1,0)+18748800; R2 = true;}
  //stop at end of current session
  else if ((Line_Selection_Number == 1) || (Line_Selection_Number == 2)) 
    {stopline = iTime(NULL,PERIOD_D1,0)+86400; R2 = false;}
  //stop at chart right border
  else if (Line_Selection_Number == 4) {stopline = startline+ Period()*60; R2 = true;}   
                             
  //Draw lines   
  ObjectCreate(line, OBJ_TREND, 0, startline, level, stopline, level);    
  ObjectSet(line, OBJPROP_STYLE, a);
  ObjectSet(line, OBJPROP_COLOR, Color);
  ObjectSet(line, OBJPROP_WIDTH, c);      
  if(!Lines_Subodinated && StringSubstr(text, 0, 4) != "_LVL") 
    {ObjectSet(line, OBJPROP_BACK, false);} 
  else {ObjectSet(line, OBJPROP_BACK, true);} 
  ObjectSet(line, OBJPROP_RAY, R2);
   
  //Labels=====================================================================================
  //Exit if label not to be shown 
  if ((!Levels_LVL_Labels_Shown) && (StringSubstr(text, 0, 4) == "_LVL")) {return (-1);}        
  else if ((!Pivot_DPV_Label_Shown) && (StringSubstr(text, 0, 2) == "PV")) {return (-1);}
  else if ((!Pivot_SR_Labels_Shown) && ((StringSubstr(text, 1, 1) == "R") || 
          (StringSubstr(text, 1, 1) == "S"))) {return (-1);}
  else if ((!Pivot_Mid_Labels_Shown) && (StringSubstr(text, 0, 1) == "M")) {return (-1);} 
  else if ((!Range_Day_HL_Labels_Shown) && (StringSubstr(text, 0, 2) == "RD")) {return (-1);}
  else if ((!Range_Week_HL_Labels_Shown) && (StringSubstr(text, 0, 2) == "RW")) {return (-1);} 
  else 
    {      
    //Name label  
    linelabel = "[CP Art]  "+StringTrimLeft(text)+" Label "+DoubleToStr(level,Digits);  
      
    //Levels: Remove "_" from Levels text
    if (text == "_LVL") {text = "LVL";} 
      
    //Pivots: Remove main pivot number
    if(StringSubstr(text, 1, 1) == "P") 
      {
      text = StringSubstr(text, 0, 3); 
      }
            
    //Pivots: adjust for inclusion of asterisk     
    if((StringSubstr(text, 0, 1) == "D") || (StringSubstr(text, 0, 1) == "F") || 
      (StringSubstr(text, 0, 1) == "M") || (StringSubstr(text, 0, 1) == "P"))
      {
      text = StringSubstr(text, 0, 3); 
      //Append asterisk when shifted pivots are displayed      
      if((TzPivots_On)&&(Daily_Main_Pivot_On || Daily_SR_Pivot_Lines_On)) 
        {text = "   " + text + "* ";}
      //Otherwise align as though there was an asterisk 
      else {text = "   " + text + "   ";}  
      } 
                 
    //Week Range: Align slightly wide labels as though there was an asterisk 
    else if (StringSubstr(text, 1, 1) == "W") 
      {
      text = "   " + text + "  ";
      } 
                                    
    //All Remaining:  Align as though there was an asterisk  
    else {text = "   " + text + "   ";}
      
    //Align labels depending on if showing prices and on digits in price      
    if(Line_Labels_Prices && StrToInteger(text)==0)       
      {         
      //Adjust for the slightly wide display of week range labels
      if(StringSubstr(text, 3, 1) == "W") 
        {
        text = text + " ";
        }                  
      if(Use_For_Forex) 
        {
        Price=DoubleToStr(level, fxDigits);
        if (Display_5_Digits_Forex) {text = "            " + text + Price;}  
        else {text = "          " + text + Price;}           
        }         
      else 
        {
        Price=DoubleToStr(level, Digits);
        spc = "       "; for (k= 1; k<= Digits; k++) {spc = spc + " ";} 
        text = spc + text + Price;
        }
      }     
 
    //Where to put labels
    if((Line_Selection_Number == 0) || (Line_Selection_Number == 1)) //start mid chart                                
      {
      spc = ""; //00 
      startlabel = Time[WindowFirstVisibleBar()/2];
      }             
    else if (Line_Selection_Number == 2 || Line_Selection_Number == 3)
      {            
      if(Time[WindowFirstVisibleBar()] < iTime(NULL, PERIOD_D1, 0)) //start at day separator
        {
        spc="       "; //07                           
        startlabel= iTime(NULL, PERIOD_D1, 0);  
        }          
      else //start start mid chart     
        {
        spc = ""; //00  
        startlabel = Time[WindowFirstVisibleBar()/2];                         
        } 
      }                       
    else if (Line_Selection_Number == 4) //start at current candle   
      {
      if(Shift_Line_Right != 0)
        {
        spc="         "; //09                   
        startlabel = Time[0] + T4;          
        }         
      else
        {
        spc="         "; //09                   
        startlabel = Time[0];   
        }                  
      }
                                     
    //Draw labels                            
    ObjectCreate(linelabel, OBJ_TEXT, 0, startlabel, level);     
    ObjectSetText(linelabel, spc + text, Line_Labels_FontSize, 
      Line_Labels_FontStyle, Line_Labels_Color);
    if(Line_Labels_Subodinated) {ObjectSet(linelabel, OBJPROP_BACK, true);} 
    else {ObjectSet(linelabel, OBJPROP_BACK, false);}
    }
  }
   
//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Today and Tomorrow Separators                                        |          
//+-------------------------------------------------------------------------------------------+   
void CurrentSeparators(string text, color Color, int linestyle, int thickness) 
  {
  a = linestyle;
  b = thickness;
  c =1; if (a==0)c=b; 
      
  //Define datetime for start of current session vline separator
  if(text == "Today")       
    { 
    dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + 
        TimeDay(iTime(NULL,PERIOD_D1,0))+ "." + 0 + ":" + "0";                                        
    T3 = StrToTime(dt)+60;            
    vline  = "[CP Art] Today Session Line";     
    } 
      
  //Define datetime for end of current session vline separator
  else                
    {
    dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + 
        TimeDay(iTime(NULL,PERIOD_D1,0))+ "." + 24 + ":" + "60";                                        
    T3 = StrToTime(dt)+60;              
    vline  = "[CP Art] Tomorrow Session Line";            
    }
                        
  if(!Separators_Thru_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, T3, 0, T3, 100);}  
  else {ObjectCreate(vline, OBJ_VLINE, 0, T3, 0);}
  ObjectSet(vline, OBJPROP_STYLE, a);    
  ObjectSet(vline, OBJPROP_WIDTH, c);
  ObjectSet(vline, OBJPROP_COLOR, Color);
  ObjectSet(vline, OBJPROP_BACK, true); 
  }
      
//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Prior Sessions Separators                                            |          
//+-------------------------------------------------------------------------------------------+   
void PriorSeparators(int i, color Color, int linestyle, int thickness)
  {
  a = linestyle;
  b = thickness;
  c =1; if (a==0)c=b;   
  vline  = "[CP Art] Prior Session  Line " + (i-1);       
                                 
  if(!Separators_Thru_SubWindows) 
    {ObjectCreate(vline, OBJ_TREND, 0, Time[i], 0, Time[i], 100);} 
  else {ObjectCreate(vline, OBJ_VLINE, 0, Time[i], 0);}     
  ObjectSet(vline, OBJPROP_STYLE, a);    
  ObjectSet(vline, OBJPROP_WIDTH, c);
  ObjectSet(vline, OBJPROP_COLOR, Color);
  ObjectSet(vline, OBJPROP_BACK, true);  
  }  

//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Market Open Lines And Labels For Current Session                     |          
//+-------------------------------------------------------------------------------------------+   
void OpenToday(string text,double time,color Color,int linestyle,int thickness,double level) 
  {
  vline  = "[CP Art]  " + StringTrimLeft(text) + " Current Session Line";    
  vlabel = "[CP Art]  " + StringTrimLeft(text) + " Current Session Label";  

  a = linestyle;
  b = thickness;
  c =1; if (a==0)c=b;
   
  time2 = time;
  if(time2<0) {time2 = time2 + 24;}
  time3 = MathFloor(time2);             
  dt= TimeYear(iTime(NULL,0,0))+ "." +TimeMonth(iTime(NULL,0,0))+ "." + 
      TimeDay(iTime(NULL,1440,0))+ "." +time3+":"+"00";           
  time4 = StrToTime(dt);      
  if(time2 != time3) {time4 = time4 + 1800;}  //add half hour  
                             
  if(!vLines_Thru_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, time4, 0, time4, 100);}  
  else {ObjectCreate(vline, OBJ_VLINE, 0, time4, 0);}
  ObjectSet(vline, OBJPROP_STYLE, a);    
  ObjectSet(vline, OBJPROP_WIDTH, c);
  ObjectSet(vline, OBJPROP_COLOR, Color);
  ObjectSet(vline, OBJPROP_BACK, true); 

  //Draw labels  
  if((Show_vLabels) && (!text == "day") && (!text == "day ")) 
    {                                   
    ObjectCreate (vlabel, OBJ_TEXT, 0, time4, level);
    ObjectSetText(vlabel, text, vLabels_FontSize, vLabels_FontStyle, vLabels_Color);
    ObjectSet(vlabel, OBJPROP_BACK, false);      
    }
  }
   
//+-------------------------------------------------------------------------------------------+
//| Subroutine to Create Market Open Lines And Labels For Prior Sessions                      |          
//+-------------------------------------------------------------------------------------------+   
void OpenPrior(int i, string text, color Color, int linestyle, int thickness, double level) 
  {   
  vline  = "[CP Art]  " + StringTrimLeft(text) + " Prior Session  " + (i-1) + " Line";    
  vlabel = "[CP Art]  " + StringTrimLeft(text) + " Prior Session  " + (i-1) + " Label"; 

  a = linestyle;
  b = thickness;
  c =1; if (a==0)c=b;
                                 
  //ObjectDelete(vline); 
  if(!vLines_Thru_SubWindows) {ObjectCreate(vline, OBJ_TREND, 0, Time[i], 0, Time[i], 100);} 
  else {ObjectCreate(vline, OBJ_VLINE, 0, Time[i], 0);}     
  ObjectSet(vline, OBJPROP_STYLE, a);    
  ObjectSet(vline, OBJPROP_WIDTH, c);
  ObjectSet(vline, OBJPROP_COLOR, Color);
  ObjectSet(vline, OBJPROP_BACK, true); 
  
  //Draw labels  
  if(Show_vLabels) 
    {                                           
    ObjectCreate (vlabel, OBJ_TEXT, 0, Time[i], level);
    ObjectSetText(vlabel, text, vLabels_FontSize, vLabels_FontStyle, vLabels_Color);
    ObjectSet(vlabel, OBJPROP_BACK, false);         
    }    
  }

//+-------------------------------------------------------------------------------------------+
//| TzShift sub-routine computes index of first/last bars of yesterday & first bar  of today  |
//+-------------------------------------------------------------------------------------------+
void TzShift(int tzServer, int tzChoice, int &idxfirstbartoday, int &idxfirstbaryesterday,
  int &idxlastbaryesterday)              
  {      
  int tzdiff               = tzServer - tzChoice,
      tzdiffsec            = tzdiff*3600, 
      barsperday           = (1440/PERIOD_H1),      
      dayofweektoday       = TimeDayOfWeek(iTime(NULL, PERIOD_H1, 0) - tzdiffsec),
      dayofweektofind      = -1;     
                
  //Due to gaps in the data, and shift of time around weekends (due to time zone)
  //it is not as easy as to just look back for a bar with 00:00 time  
  idxfirstbartoday= 0; idxfirstbaryesterday= 0;  idxlastbaryesterday= 0;     
  switch (dayofweektoday) { 
     case 0: dayofweektofind = 5; break; // Sun (Sunday considered as short trading day)
     default: dayofweektofind = dayofweektoday -1; break;}                    

  //Search backwards for the last occurence of a "today's first bar"
  for(int i=1; i<=barsperday + 1; i++){
    datetime timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
      if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
  for(int j= 0; j<=2*barsperday + 1; j++) {
    datetime timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
    if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
  for(j= 1; j<=barsperday; j++) {
    datetime timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
    if(TimeDayOfWeek(timey2) != dayofweektofind) 
      {idxfirstbaryesterday = idxlastbaryesterday+j-1; break;}}
      
//Start - Added by CaveMan to cater for missing/partial previous day's data        
  //Search for Friday's data range if yesterday is a Sunday  
  if(dayofweektofind == 0)
    {
    //Search backwards for the last occurence of a "today's first bar"
    for(i=1; i<=barsperday + 1; i++){
       timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
       if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
    for(j= 0; j<=2*barsperday + 1; j++) {
       timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
       if(TimeDayOfWeek(timey) == 5) {idxlastbaryesterday = i+j; break;}}
    for(j= 1; j<=barsperday; j++) {
       timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
       if(TimeDayOfWeek(timey2) != 5){idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
    } 
                      
  //This section takes care of the Christmas and New Year Day problem where there is a day 
  //of partial data on the eve of these holidays followed by an entire day without any data  
  //at all. The workaround to handle this on market open after the holiday is to use data 
  //from the day before the eve of the holiday right up to the end of the eve of the holiday 
  //for pivot computation. E.g. On the 26 December market opens after Christmas. Use full 
  //day data of 23rd plus partial data of 24th Dec to compute the pivots. Same applies to 
  //New Year Day.
  int YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
  if(YesterdayBarsTotal == 0)   
    {
    if (dayofweektofind == 0) {dayofweektofind = 4;} 
    else if (dayofweektofind == 1) {dayofweektofind = 5;} 
    else {dayofweektofind = dayofweektofind - 1;}

    //Repeat this search with the new dayofweektofind information
    for(i=1; i<=barsperday + 1; i++) {
       timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
       if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
    for(j= 0; j<=2*barsperday + 1; j++) {
       timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
       if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
    for(j= 1; j<=3*barsperday; j++) {
       timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
       if(TimeDayOfWeek(timey2) != dayofweektofind) 
         {idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
    YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
    }            
  //Standard codes to handle days of partial data (including Fridays and Sundays)
  //If one hour bars < 24, use additional data from one more day back 
  if(YesterdayBarsTotal < 23)
    {
    //for Sunday, search Thursday and found Thursday partial data, then search Wednesday. 
    if (dayofweektoday == 0 && dayofweektofind == 4) {dayofweektofind = 3;} 
    //for Monday, Friday no data then include Thursday and Sunday data.
    else if (dayofweektoday == 1 && dayofweektofind == 4) {dayofweektofind = 4;} 
    else if (dayofweektofind == 0 || dayofweektofind == 1) { dayofweektofind = 5;} 
    else {dayofweektofind = dayofweektofind - 1;}
    //Search backwards for the last occurence of a "today's first bar"
    for(i=1; i<=barsperday + 1; i++) {
      timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
      if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
    for(j= 0; j<=3*barsperday + 1; j++) { // Need to increase another 24 hours to search
      timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
      if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
    for(j= 1; j<=4*barsperday; j++) {
      timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
      if(TimeDayOfWeek(timey2) != dayofweektofind)
        {idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}                             
    //This small section handles a very unique combination of events that coincide during the 
    //short opening trading hours of the Sunday following Christmas Day and New Year Day if 
    //both of these holidays happen to fall on a Thursday.  This occured on the short Sunday 
    //trading hours before Monday kicks in. For those few hours, pivot used on Friday are 
    //extended into Sunday. These pivot levels are calculated using full Thursday's data. In 
    //this unique situation, Thursday was New Year Day (major public holiday)with no data at 
    //all. This will cause TzPivot to malfunction in similar situation in the future. As a 
    //workaround, for those short few hours of Sunday, if a similar situation should occur,
    //partial data of Wednesday and Friday will be used for pivot computation.  
    YesterdayBarsTotal = idxfirstbaryesterday - idxlastbaryesterday;
    if(YesterdayBarsTotal == 0)   
      {
      if(dayofweektofind == 1) {dayofweektofind = 5;}
      else{dayofweektofind = dayofweektofind - 1;}
      //Repeat this search with the new dayofweektofind information
      for(i=1; i<=barsperday + 1; i++) {
        timet = iTime(NULL, PERIOD_H1, i) - tzdiffsec;
        if(TimeDayOfWeek(timet) != dayofweektoday) {idxfirstbartoday = i-1;break;}}
      for(j= 0; j<=2*barsperday + 1; j++) {
        timey = iTime(NULL, PERIOD_H1, i+j) - tzdiffsec;
        if(TimeDayOfWeek(timey) == dayofweektofind) {idxlastbaryesterday = i+j; break;}}
      for(j= 1; j<=3*barsperday; j++) {
        timey2 = iTime(NULL, PERIOD_H1, idxlastbaryesterday+j) - tzdiffsec;
        if(TimeDayOfWeek(timey2) != dayofweektofind)
          {idxfirstbaryesterday = idxlastbaryesterday+j-1;break;}}
      idxlastbaryesterday = idxfirstbartoday + 1;
         }                     
    //Include partial day's data and combine with previous day's data for pivot computation. 
    //Do not include for Sundays as Sunday short trading hours uses Friday's pivots that are 
    //computed from full Thursday's data.  This is more for handling of holidays including 
    //Christmas and New Year Day. . 
    if (dayofweektoday != 0) idxlastbaryesterday = idxfirstbartoday + 1; 
    }          
//End - Added by CaveMan to cater for missing/partial previous day's data 
  }                                

//+-------------------------------------------------------------------------------------------+
//| Subroutine to make city and time labels                                                   |
//+-------------------------------------------------------------------------------------------+
int MakeLabel( string n, int xoff, int yoff )
  {
  if(ObjectFind(n) != 0)
    {    
    ObjectCreate( n, OBJ_LABEL, 0, 0, 0 );
    ObjectSet( n, OBJPROP_CORNER, 0 );
    ObjectSet( n, OBJPROP_XDISTANCE, xoff );
    ObjectSet( n, OBJPROP_YDISTANCE, yoff );
    ObjectSet( n, OBJPROP_BACK, false ); 
    }
  else{ObjectMove(n, 0, xoff, yoff);}       
  } 

//+-------------------------------------------------------------------------------------------+
//| Subroutine to convert time to a string for time labels                                    |
//+-------------------------------------------------------------------------------------------+
string TimeToString( datetime when )
   {
   hour = TimeHour( when );
   if ( !Show_AMPM_Time ) 
      {
      timeStr = (TimeToStr( when, TIME_MINUTES));
      }

   // User wants 12HourTime format with "AM" or "PM".   
   // FYI, if >12:00, subtract 12 hours in seconds which is 12*60*60=43200
   else
      {
      if ( hour >  12 || hour == 0) timeStr = TimeToStr( (when - 43200), TIME_MINUTES);
      else timeStr = TimeToStr( when, TIME_MINUTES);
      if ( hour >= 12) timeStr = StringConcatenate(timeStr, " PM");
      else timeStr = StringConcatenate(timeStr, " AM");
      }
   return (timeStr);
   }
   
//+-------------------------------------------------------------------------------------------+
//| Subroutine to retrieve date and time from time array                                      |
//+-------------------------------------------------------------------------------------------+
datetime TimeArrayToTime(int& LocalTimeArray[])
  {  
  nYear=LocalTimeArray[0]&0x0000FFFF;
  nMonth=LocalTimeArray[0]>>16;
  nDay=LocalTimeArray[1]>>16;
  nHour=LocalTimeArray[2]&0x0000FFFF;
  nMin=LocalTimeArray[2]>>16;
  nSec=LocalTimeArray[3]&0x0000FFFF;
  nMilliSec=LocalTimeArray[3]>>16;
  LocalTimeS = FormatDateTime(nYear,nMonth,nDay,nHour,nMin,nSec);
  Local_Time = StrToTime( LocalTimeS );
  return(Local_Time);
  }   

//+-------------------------------------------------------------------------------------------+
//| Subroutine to format time labels                                                          |
//+-------------------------------------------------------------------------------------------+
string FormatDateTime(int nYear,int nMonth,int nDay,int nHour,int nMin,int nSec)
  {
  sMonth=100+nMonth;
  sMonth=StringSubstr(sMonth,1);
  sDay=100+nDay;
  sDay=StringSubstr(sDay,1);
  sHour=100+nHour;
  sHour=StringSubstr(sHour,1);
  sMin=100+nMin;
  sMin=StringSubstr(sMin,1);
  sSec=100+nSec;
  sSec=StringSubstr(sSec,1);
  return(StringConcatenate(nYear,".",sMonth,".",sDay," ",sHour,":",sMin,":",sSec));
  }    

//+-------------------------------------------------------------------------------------------+
//| Subroutine to get time zone information for each city                                     |
//+-------------------------------------------------------------------------------------------+
void GetAllTimeZoneInfo(int& AucklandTZInfoArray[], int& SydneyTZInfoArray[], 
   int& TokyoTZInfoArray[], int& ChinaTZInfoArray[], int& JakartaTZInfoArray[], 
   int& IndiaTZInfoArray[], int& DubaiTZInfoArray[], int& MoscowTZInfoArray[], 
   int& IsraelTZInfoArray[], int& HelsinkiTZInfoArray[], int& BerlinTZInfoArray[], 
   int& LondonTZInfoArray[], int& BrazilTZInfoArray[], int& NewYorkTZInfoArray[], 
   int& CentralTZInfoArray[], int& MexicoTZInfoArray[], int& MountainTZInfoArray[], 
   int& PacificTZInfoArray[])
   
   {
   AucklandTZInfoArray[0]  = -720;
   AucklandTZInfoArray[17] = 262144; //4<<16 == 262144  April
   AucklandTZInfoArray[18] = 65536;  //1<<16 == 65536   1st Sunday
   AucklandTZInfoArray[19] = 3;
   AucklandTZInfoArray[20] = 0;
   AucklandTZInfoArray[21] = 0;
   AucklandTZInfoArray[38] = 589824; //9<<16 == 589824  September
   AucklandTZInfoArray[39] = 327680; //5<<16 == 327680  5th/Last Sunday
   AucklandTZInfoArray[40] = 2;
   AucklandTZInfoArray[41] = 0;
   AucklandTZInfoArray[42] = -60;
   
   SydneyTZInfoArray[0]  = -600;
   SydneyTZInfoArray[17] = 262144; //4<<16 == 262144  April
   SydneyTZInfoArray[18] = 65536;  //1<<16 == 65536   1st Sunday
   SydneyTZInfoArray[19] = 3;
   SydneyTZInfoArray[20] = 0;
   SydneyTZInfoArray[21] = 0;
   SydneyTZInfoArray[38] = 655360; //10<<16 == 655360  October
   SydneyTZInfoArray[39] = 65536;  //1<<16 == 65536    1st Sunday
   SydneyTZInfoArray[40] = 2;
   SydneyTZInfoArray[41] = 0;
   SydneyTZInfoArray[42] = -60;
   
   // FYI Tokyo = Seoul
   TokyoTZInfoArray[0]  = -540;
   TokyoTZInfoArray[17] = 0; 
   TokyoTZInfoArray[18] = 0;
   TokyoTZInfoArray[19] = 0;
   TokyoTZInfoArray[20] = 0;
   TokyoTZInfoArray[21] = 0;
   TokyoTZInfoArray[38] = 0;
   TokyoTZInfoArray[39] = 0;
   TokyoTZInfoArray[40] = 0;
   TokyoTZInfoArray[41] = 0;
   TokyoTZInfoArray[42] = 0;
   
   // FYI, Beijing = Perth, Singapore, Taipei, Hong Kong
   ArrayCopy(ChinaTZInfoArray, TokyoTZInfoArray);
   ChinaTZInfoArray[0] = -480;
   
   // FYI, Jakarta = Bangkok
   ArrayCopy(JakartaTZInfoArray, TokyoTZInfoArray);
   JakartaTZInfoArray[0] = -420;
   
   ArrayCopy(IndiaTZInfoArray, TokyoTZInfoArray);
   IndiaTZInfoArray[0]   = -330;  // NOTE! Top of the hour is 30 min off most world timezones.
   
   ArrayCopy(DubaiTZInfoArray, TokyoTZInfoArray);
   DubaiTZInfoArray[0]   = -240;
 
   IsraelTZInfoArray[0]  = -120;
   IsraelTZInfoArray[17] = 589824;  //9<<16 == 589824  September
   IsraelTZInfoArray[18] = 131072;  //2<<16  = 131072
   IsraelTZInfoArray[19] = 2;
   IsraelTZInfoArray[20] = 0;
   IsraelTZInfoArray[21] = 0;
   IsraelTZInfoArray[38] = 196608; //3<<16 == 196608  March
   IsraelTZInfoArray[39] = 327685; //5<<16  = 327680 AND "5" (*IF* Sunday is "0", is "5" Friday??)
   IsraelTZInfoArray[40] = 2;
   IsraelTZInfoArray[41] = 0;
   IsraelTZInfoArray[42] = -60;
  
   LondonTZInfoArray[0]  = 0;
   LondonTZInfoArray[17] = 655360; //10<<16 == 655360  October
   LondonTZInfoArray[18] = 327680; //5<<16 == 327680.  5th/Last Sunday. 
   LondonTZInfoArray[19] = 2;
   LondonTZInfoArray[20] = 0;
   LondonTZInfoArray[21] = 0;
   LondonTZInfoArray[38] = 196608; //3<<16 == 196608  March
   LondonTZInfoArray[39] = 327680; //5<<16 == 327680  5th/Last Sunday
   LondonTZInfoArray[40] = 1;
   LondonTZInfoArray[41] = 0;
   LondonTZInfoArray[42] = -60;
   
   ArrayCopy(MoscowTZInfoArray, LondonTZInfoArray);
   MoscowTZInfoArray[0]  = -180;
   MoscowTZInfoArray[19] = 3;
   MoscowTZInfoArray[40] = 2;
   
   // FYI, Helsinki = Athens
   ArrayCopy(HelsinkiTZInfoArray, LondonTZInfoArray);
   HelsinkiTZInfoArray[0]  = -120;
   HelsinkiTZInfoArray[19] = 4;
   HelsinkiTZInfoArray[40] = 3;
   
   // FYI, Berlin = Belgrade, Brussels, Paris, Sarajevo
   ArrayCopy(BerlinTZInfoArray, MoscowTZInfoArray);
   BerlinTZInfoArray[0] = -60;
   
   // NOTE! Brazil's ST/DST is likely a hardcoded date which will need updating EVERY YEAR!
   BrazilTZInfoArray[0]  = 180;
   BrazilTZInfoArray[17] = 131072;   //2<<16  = 131072
   BrazilTZInfoArray[18] = 196614;   //3<<16 == 196608 AND 6, so, Saturday?? or Mar 6?
   BrazilTZInfoArray[19] = 3866647;
   BrazilTZInfoArray[20] = 65470523;
   BrazilTZInfoArray[21] = 0;
   BrazilTZInfoArray[38] = 655360;   //10<<16 == 655360
   BrazilTZInfoArray[39] = 196614;
   BrazilTZInfoArray[40] = 3866647;
   BrazilTZInfoArray[41] = 65470523;
   BrazilTZInfoArray[42] = -60;
   
   //ArrayCopy(BuenasAriesTZInfoArray, BuenasAriesTZInfoArray);
   //BuenasAriesTZInfoArray[0] = 180;
   
   NewYorkTZInfoArray[0]  = 300;
   NewYorkTZInfoArray[17] = 720896; //wYear = 0. wMonth = 11, and 11<<16 == 720896
   NewYorkTZInfoArray[18] = 65536;  //wDOW = 0 = Sunday. nDay = 1 and 1<<16 == 65536 
   NewYorkTZInfoArray[19] = 2;
   NewYorkTZInfoArray[20] = 0;
   NewYorkTZInfoArray[21] = 0;
   NewYorkTZInfoArray[38] = 196608; //3<<16 == 196608  March
   NewYorkTZInfoArray[39] = 131072; //2<<16 == 131072  2nd Sunday
   NewYorkTZInfoArray[40] = 2;
   NewYorkTZInfoArray[41] = 0;
   NewYorkTZInfoArray[42] = -60;
   
   ArrayCopy(CentralTZInfoArray, NewYorkTZInfoArray);
   CentralTZInfoArray[0] = 360;
 
   MexicoTZInfoArray[0]  = 360;
   MexicoTZInfoArray[17] = 655360; //10<<16 == 655360  October
   MexicoTZInfoArray[18] = 327680; //5<<16 == 327680  5th/Last Sunday
   MexicoTZInfoArray[19] = 2;
   MexicoTZInfoArray[20] = 0;
   MexicoTZInfoArray[21] = 0;
   MexicoTZInfoArray[38] = 262144; //4<<16 == 262144  April
   MexicoTZInfoArray[39] = 65536;  //1<<16 == 65536   1st Sunday
   MexicoTZInfoArray[40] = 2;
   MexicoTZInfoArray[41] = 0;
   MexicoTZInfoArray[42] = -60;
   
   ArrayCopy(MountainTZInfoArray, NewYorkTZInfoArray);
   MountainTZInfoArray[0] = 420;
   
   ArrayCopy(PacificTZInfoArray, NewYorkTZInfoArray);
   PacificTZInfoArray[0] = 480;   
   } 
   
//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+      

