//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                               SonicR Line Dragon-Trend.mq4                                |
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2011 traderathome"
#property link      "email: traderathome@msn.com"

/*---------------------------------------------------------------------------------------------
Overview, _v2:

This indicator displays both the SonicR Dragon and the SonicR Trend.  The SonicR Dragon is 
based upon 34 EMA averaging of close prices with high/low prices defining the outer edges.
The SonicR Trend is based upon 89 EMA averaging of close prices.  The Dragon centerline and 
the Trend line can be constructed with a wide background line and a narrow accent top line.
 
Optional Displays of the Dragon and Trend-
   The Dragon and Trend from one TF chart can be displayed on a different TF chart. 
   1. The M5 display of the Dragon and Trend to be applied to the M15 TF chart.
   2. The M15 display of the Dragon and Trend be applied to the M30 and H1 TF charts.
   3. The H1 display of the Dragon and Trend be applied to the H4 TF chart.
   By placing the cursor on either the Dragon or Trend and a window appears identifying  
   this indicator.  Right click the cursor and another window appears identifying the
   configurations of the Dragon and Trend that have been applied to the chart.  Brokers 
   that have short Sunday sessions lack data to have such "transferred" configurations 
   drawn seamlessly.  This indicator identifies when that is the case and compensates by 
   filling with data that is the average of that which is before and after an interruption 
   of the draw.  If this not take place automatically when a chart is loaded, or selected, 
   simply changing the chart TF once should refresh the draw correctly. 
   
You can select a range of TFs for the display of this indicator, so it automatically will not 
display on a chart TF outside that range.  The indicator can be turned on/off without having 
to remove it from the chart, thereby preserving your chart settings.  

Changes from previous release 04-04-2012 to current release 05-01-2012:
01 - Compensate for data dropouts interrupting the draw of transferred configurations.

                                        
                                                                    - Traderathome, 05-01-2012  
----------------------------------------------------------------------------------------------- 
Suggested Settings:   White Chart        w/VSA Candles      Black Chart       w/VSA Candles

Dragon-
indicator_color1      DeepSkyBlue        DeepSkyBlue        C'91,147,191'     SlateBlue  
indicator_color2      DeepSkyBlue        DeepSkyBlue        C'91,147,191'     SlateBlue  
indicator_color3      LightSlateGray     LightSkyBlue       C'105,120,135'    DarkSlateBlue  
indicator_color4      MediumSpringGreen  CLR_NONE           C'47,88,121'      CLR_NONE  
indicator_width1      1                  1                  1                 1
indicator_width2      1                  1                  1                 1
indicator_width3      4                  2                  4                 2
indicator_width4      2                  1                  2                 1
indicator_style1      STYLE_SOLID        STYLE_SOLID        STYLE_SOLID       STYLE_SOLID
indicator_style2      STYLE_SOLID        STYLE_SOLID        STYLE_SOLID       STYLE_SOLID
indicator_style3      STYLE_SOLID        STYLE_SOLID        STYLE_SOLID       STYLE_SOLID
indicator_style4      STYLE_SOLID        STYLE_SOLID        STYLE_SOLID       STYLE_SOLID

Trend-
indicator_color5      Pink               CLR_NONE           CLR_NONE          CLR_NONE
indicator_color6      Purple             Magenta            C'215,0,215'      C'215,0,215'
indicator_width5      2                  1                  1                 1
indicator_width6      1                  1                  1                 1
indicator_style5      STYLE_SOLID        STYLE_SOLID        STYLE_SOLID       STYLE_SOLID
indicator_style6      STYLE_SOLID        STYLE_SOLID        STYLE_SOLID       STYLE_SOLID                                                                                      
---------------------------------------------------------------------------------------------*/


//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+ 
#property indicator_chart_window 
#property indicator_buffers  6

//Dragon
#property indicator_color1   DeepSkyBlue 
#property indicator_color2   DeepSkyBlue
#property indicator_color3   LightSkyBlue    
#property indicator_color4   CLR_NONE     
#property indicator_width1   1
#property indicator_width2   1
#property indicator_width3   2
#property indicator_width4   1
#property indicator_style1   STYLE_SOLID
#property indicator_style2   STYLE_SOLID
#property indicator_style3   STYLE_SOLID
#property indicator_style4   STYLE_SOLID

//Trend
#property indicator_color5   CLR_NONE   
#property indicator_color6   Magenta     
#property indicator_width5   1
#property indicator_width6   1
#property indicator_style5   STYLE_SOLID
#property indicator_style6   STYLE_SOLID

//global external inputs
extern bool   Indicator_On                    = false;
extern bool   Dragon_On                       = true;
extern bool   Trend_On                        = true;
extern int    Indicator_Display_Min_TF        = 1;
extern int    Indicator_Display_Max_TF        = 43200;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";

extern string __                              = "";
extern string Part_2                          = "Dragon-Trend Display Options:";
extern bool   M5_On_M15                       = false;
extern bool   M15_On_M30                      = false;
extern bool   M15_On_H1                       = false;
extern bool   H1_On_H4                        = false; 

//Global Buffers and Variables
bool    FLAG_deinit;
int     i, x, counted_bars, limit, BarShift;

//Dragon section
double  DragonUpper[], DragonLower[], DragonCntrBot[], DragonCntrTop[];
string  Dragon_MA_String              = "EMA";
int     Dragon_MA_Type                = 1;
int     Dragon_MA_Period              = 34;
bool    Dragon_Exp_vs_Simple          = true;

//Trend section
double  TrendBot[], TrendTop[];
string  Trend_MA_String               = "EMA";
int     Trend_MA_Type                 = 1;
int     Trend_MA_Period               = 89;
bool    Trend_Exp_vs_Simple           = true; 

/*General notes:
MAType   = 0=SMA,1=EMA,2=SMMA,3=LWMA
MAPrice  = 0=CLOSE,1=OPEN,2=HIGH,3=LOW,4=MEDIAN,5=PP,6=WEIGHT
*/

//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+ 
int init()
  {
  FLAG_deinit = false; 
  
  //Indicator              
  if(Dragon_On)
    {     
    SetIndexBuffer(0, DragonUpper);
    SetIndexBuffer(1, DragonLower);
    SetIndexBuffer(2, DragonCntrBot);
    SetIndexBuffer(3, DragonCntrTop);                     
    SetIndexStyle(0, DRAW_LINE);
    SetIndexStyle(1, DRAW_LINE);
    SetIndexStyle(2, DRAW_LINE);
    SetIndexStyle(3, DRAW_LINE);            
    SetIndexEmptyValue(0,0);
    SetIndexEmptyValue(1,0);
    SetIndexEmptyValue(2,0);
    SetIndexEmptyValue(3,0);     
    }    
  if(Trend_On)
    {          
    SetIndexBuffer(4, TrendBot);
    SetIndexBuffer(5, TrendTop);
    SetIndexStyle(4, DRAW_LINE);   
    SetIndexStyle(5, DRAW_LINE);
    SetIndexEmptyValue(4,0);
    SetIndexEmptyValue(5,0);
    }
   
  //Indicator ShortName    
  if(Dragon_On && Trend_On)
    {
    if(M5_On_M15 && Period() == 15) 
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( M5  "+Dragon_MA_Period+" "+Dragon_MA_String
      +",  M5  "+Trend_MA_Period+" "+ Trend_MA_String+" )  ");
      }
    else if((M15_On_M30 && Period() == 30) ||
      (M15_On_H1 && Period() == 60))
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( M15  "+Dragon_MA_Period+" "+Dragon_MA_String
      +",  M15  "+Trend_MA_Period+" "+ Trend_MA_String+" )  ");
      } 
    else if(H1_On_H4 && Period() == 240)
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( H1  "+Dragon_MA_Period+" "+Dragon_MA_String
      +",  H1  "+Trend_MA_Period+" "+ Trend_MA_String+" )  ");
      }     
    else
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( "+Dragon_MA_Period+" "+Dragon_MA_String
      +",  "+Trend_MA_Period+" "+ Trend_MA_String+" )  ");
      }         
    }
  else if(Dragon_On && !Trend_On)
    {
    if(M5_On_M15 && Period() == 15)
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( M5  "+Dragon_MA_Period+" "+Dragon_MA_String+" )  ");
      }
    else if((M15_On_M30 && Period() == 30) ||
      (M15_On_H1 && Period() == 60))
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( M15  "+Dragon_MA_Period+" "+Dragon_MA_String+" )  ");
      } 
    else if(H1_On_H4 && Period() == 240)
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( H1  "+Dragon_MA_Period+" "+Dragon_MA_String+" )  ");
      }
    else
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( "+Dragon_MA_Period+" "+Dragon_MA_String+" )  ");
      }           
    }    
  else if(Trend_On && !Dragon_On)
    {
    if(M5_On_M15 && Period() == 15)
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( M5  "+Trend_MA_Period+" "+Trend_MA_String+" )  ");
      }
    else if((M15_On_M30 && Period() == 30) ||
      (M15_On_H1 && Period() == 60))
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( M15  "+Trend_MA_Period+" "+Trend_MA_String+" )  ");
      }
    else if(H1_On_H4 && Period() == 240)
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( H1  "+Trend_MA_Period+" "+Trend_MA_String+" )  ");
      }
    else
      {
      IndicatorShortName("SonicR Line Dragon-Trend  ( "+Trend_MA_Period+" "+ Trend_MA_String+" )  ");
      }           
    }  
                                      
  return(0);
  }
   
//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+ 
int deinit()
  {       
  return(0);
  }
   
//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+ 
int start()
  {
  //If indicator is "Off" or chart TF is out of range deinitialize only once, not every tick.  
  if(!Indicator_On||Period()<Indicator_Display_Min_TF||Period()>Indicator_Display_Max_TF)    
    {
    if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
    return(0);
    }

  //Otherwise indicator is "On" & chart TF is in display range, so proceed.   
  deinit(); FLAG_deinit = false;
      
  //---- check for possible errors
  counted_bars = IndicatorCounted();   
  if(counted_bars < 0) {return(-1);}
       
  //---- last counted bar will be recounted
  if(counted_bars > 0) {counted_bars--;}  
  limit = Bars - counted_bars;
 
  if(Dragon_On)
    {    
    for(i = limit - 1; i >= 0; i--)
      {
      if(M5_On_M15 && Period() == 15)  
        { 
        BarShift = iBarShift(NULL,5,Time[i],true); 
        DragonUpper[i] = iMA(NULL,5,Dragon_MA_Period,0,Dragon_MA_Type,2,BarShift);
        DragonLower[i] = iMA(NULL,5,Dragon_MA_Period,0,Dragon_MA_Type,3,BarShift);
        DragonCntrBot[i] = iMA(NULL,5,Dragon_MA_Period,0,Dragon_MA_Type,0,BarShift);
        DragonCntrTop[i] = iMA(NULL,5,Dragon_MA_Period,0,Dragon_MA_Type,0,BarShift);
        } 
      else if((M15_On_M30 && Period() == 30)  ||  (M15_On_H1 && Period() == 60))
        { 
        BarShift = iBarShift(NULL,15,Time[i],true); 
        DragonUpper[i] = iMA(NULL,15,Dragon_MA_Period,0,Dragon_MA_Type,2,BarShift);
        DragonLower[i] = iMA(NULL,15,Dragon_MA_Period,0,Dragon_MA_Type,3,BarShift);
        DragonCntrBot[i] = iMA(NULL,15,Dragon_MA_Period,0,Dragon_MA_Type,0,BarShift);
        DragonCntrTop[i] = iMA(NULL,15,Dragon_MA_Period,0,Dragon_MA_Type,0,BarShift);
        }
      else if(H1_On_H4 && Period() == 240) 
        { 
        BarShift = iBarShift(NULL,60,Time[i],true);      
        DragonUpper[i] = iMA(NULL,60,Dragon_MA_Period,0,Dragon_MA_Type,2,BarShift);
        DragonLower[i] = iMA(NULL,60,Dragon_MA_Period,0,Dragon_MA_Type,3,BarShift);
        DragonCntrBot[i] = iMA(NULL,60,Dragon_MA_Period,0,Dragon_MA_Type,0,BarShift);
        DragonCntrTop[i] = iMA(NULL,60,Dragon_MA_Period,0,Dragon_MA_Type,0,BarShift);
        }                  
      else    
        {              
        BarShift = iBarShift(NULL,NULL,Time[i],true);                
        DragonUpper[i] = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_MA_Type,2,BarShift);
        DragonLower[i] = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_MA_Type,3,BarShift);
        DragonCntrBot[i] = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_MA_Type,0,BarShift);
        DragonCntrTop[i] = iMA(NULL,NULL,Dragon_MA_Period,0,Dragon_MA_Type,0,BarShift);
        }
      //Brokers with short Sunday sessions lack necessary data to make Dragon continuous, so when
      //there is a data "dropout" (current value<half previous value), fill with average data: 
      if(DragonUpper[i+1]< DragonUpper[i+2]/2){DragonUpper[i+1]= (DragonUpper[i+2]+ DragonUpper[i])/2;}
      if(DragonLower[i+1]< DragonLower[i+2]/2){DragonLower[i+1]= (DragonLower[i+2]+ DragonLower[i])/2;}
      if(DragonCntrBot[i+1]< DragonCntrBot[i+2]/2){DragonCntrBot[i+1]= (DragonCntrBot[i+2]+ DragonCntrBot[i])/2;}
      if(DragonCntrTop[i+1]< DragonCntrTop[i+2]/2){DragonCntrTop[i+1]= (DragonCntrTop[i+2]+ DragonCntrTop[i])/2;}          
      }
    }      
  	 
  if(Trend_On)
  	 {
  	 for(i = limit - 1; i >= 0; i--)
  	   {
  	   if(M5_On_M15 && Period() == 15)  
        {
        BarShift = iBarShift(NULL,5,Time[i],true);                 
        TrendBot[i] = iMA(NULL,5,Trend_MA_Period,0,Trend_MA_Type,0,BarShift); 
        TrendTop[i] = iMA(NULL,5,Trend_MA_Period,0,Trend_MA_Type,0,BarShift);   
        } 	    
      else if((M15_On_M30 && Period() == 30)  ||  (M15_On_H1 && Period() == 60))
        {
        BarShift = iBarShift(NULL,15,Time[i],true);                 
        TrendBot[i] = iMA(NULL,15,Trend_MA_Period,0,Trend_MA_Type,0,BarShift); 
        TrendTop[i] = iMA(NULL,15,Trend_MA_Period,0,Trend_MA_Type,0,BarShift);   
        }
      else if(H1_On_H4 && Period() == 240)
        {
        BarShift = iBarShift(NULL,60,Time[i],true);                 
        TrendBot[i] = iMA(NULL,60,Trend_MA_Period,0,Trend_MA_Type,0,BarShift); 
        TrendTop[i] = iMA(NULL,60,Trend_MA_Period,0,Trend_MA_Type,0,BarShift);   
        } 
      else
        {   
        BarShift = iBarShift(NULL,NULL,Time[i],true);                 
        TrendBot[i] = iMA(NULL,NULL,Trend_MA_Period,0,Trend_MA_Type,0,BarShift); 
        TrendTop[i] = iMA(NULL,NULL,Trend_MA_Period,0,Trend_MA_Type,0,BarShift); 
        }
      //Brokers with short Sunday sessions lack necessary data to make Trend continuous, so when
      //there is a data "dropout" (current value<half previous value), fill with average data:    
      if(TrendBot[i+1]< TrendBot[i+2]/2){TrendBot[i+1]= (TrendBot[i+2]+ TrendBot[i])/2;}
      if(TrendTop[i+1]< TrendTop[i+2]/2){TrendTop[i+1]= (TrendTop[i+2]+ TrendTop[i])/2;}                      
      }
    }
  	      
  return(0);
  }
   
//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+       

